#! /usr/bin/env python
"""
util_MassGrid.py <fnames>
    - prints out <m1> <m2> <lnL>  <neff> \sigma/L for each individual ILE *mass point*, to stdout
    - fnames can be an arbitrary mix of .sqlite or .xml files
    - currently *hardcodes* intrinsic (mass) parameter list

Used to generate ascii dumps summarizing runs
"""
import sys
import os
import RIFT.misc.xmlutils as xmlutils
from optparse import OptionParser
from glue.ligolw import lsctables, table, utils

import numpy as np
import RIFT.misc.weight_simulations as weight_simulations

# optp= OptionParser()
# opts, args = optp.parse_args()

data_at_intrinsic = {}
for fname in sys.argv[1:]:
    fExtension = fname.split(".")[-1]
    # if .sqlite extension
    if fExtension == "sqlite":
        samples = xmlutils.db_to_samples(fname, lsctables.SnglInspiralTable, ['mass1', 'mass2', 'snr', 'tau0', 'event_duration'])
        for row in samples:  # print each individual row, don't reweight yet
            if not (data_at_intrinsic.has_key((row.mass1,row.mass2))):
                data_at_intrinsic[(row.mass1,row.mass2)] = [[row.snr, row.tau0, row.event_duration]]
            else:
                data_at_intrinsic[(row.mass1,row.mass2)].append([row.snr, row.tau0, row.event_duration])
    # otherwise xml: could have just used ligolw_print
    elif fExtension == "xml" or fExtension == "gz":
        samples = table.get_table(utils.load_filename(fname), lsctables.SnglInspiralTable.tableName)
        for row in samples:  # print each individual row, don't reweight yet
            if not (data_at_intrinsic.has_key((row.mass1,row.mass2))):
                data_at_intrinsic[(row.mass1,row.mass2)] = [[row.snr, row.tau0, row.event_duration]]
            else:
                data_at_intrinsic[(row.mass1,row.mass2)].append([row.snr, row.tau0, row.event_duration])


# Loop over each key and print out
for intrinsic in data_at_intrinsic.keys():
    lnL, neff, sigmaOverL =   np.transpose(data_at_intrinsic[intrinsic])
    lnLmax = np.max(lnL)
    sigma = sigmaOverL*np.exp(lnL-lnLmax)  # remove overall Lmax factor, which factors out from the weights constructed from \sigma
    wts = weight_simulations.AverageSimulationWeights(None, None,sigma)   
    lnLmeanMinusLmax = np.log(np.sum(np.exp(lnL - lnLmax)*wts))
    sigmaNetOverL = (np.sqrt(1./np.sum(1./sigma/sigma)))/np.exp(lnLmeanMinusLmax)
    print intrinsic[0], intrinsic[1], lnLmeanMinusLmax+lnLmax, np.sum(neff), sigmaNetOverL





#! /usr/bin/env python
#
# GOAL
#   - compute I(gamma), based on intermediate calculations of G(eta, LambdaTilde)
#     [User may want to smooth/validate intermediate integral before using it!]
#   - relies on 
#      - calculating EOS
#      - 
#
# ARGUMENTS
#   - redshift (assumed known)
#   - redshifted chirp mass
#   - EOS parameters 
#
#
# EXAMPLE
#   - python `which util_MarginalizedLikelihoodEOSIntegralFromIntermediate.py`  --intermediate-file analyze_27/qt/intermediate_nospin.dat 
#   python `which util_MarginalizedLikelihoodEOSIntegralFromIntermediate.py`  --intermediate-file analyze_27/qt/intermediate_nospin.dat  --parameter gamma1 --parameter-value 0.707 --parameter gamma2 --parameter-value 0.707 --mcz 1.26940994223 --redshift `cat redshift_of_27.dat`


import RIFT.physics.EOSManager as EOSManager

import argparse
import sys
import numpy as np
import numpy.lib.recfunctions
import scipy
import RIFT.lalsimutils as lalsimutils
import lalsimulation as lalsim
import lal
import functools
import itertools

import os



from sklearn.gaussian_process import GaussianProcessRegressor
from sklearn.gaussian_process.kernels import RBF, WhiteKernel, ConstantKernel as C


def fit_gp(x,y,x0=None,symmetry_list=None,y_errors=None,hypercube_rescale=False,fname_export="gp_fit"):
    """
    x = array so x[0] , x[1], x[2] are points.
    """

    # If we are loading a fit, override everything else
    if opts.fit_save_gp and os.path.isfile(opts.fit_save_gp):
        print " WARNING: Do not re-use fits across architectures or versions : pickling is not transferrable "
        my_gp=joblib.load(opts.fit_save_gp)
        return lambda x:my_gp.predict(x)

    # Amplitude: 
    #   - We are fitting lnL.  
    #   - We know the scale more or less: more than 2 in the log is bad
    # Scale
    #   - because of strong correlations with chirp mass, the length scales can be very short
    #   - they are rarely very long, but at high mass can be long
    #   - I need to allow for a RANGE

    length_scale_est = []
    length_scale_bounds_est = []
    for indx in np.arange(len(x[0])):
        # These length scales have been tuned by expereience
        length_scale_est.append( 2*np.std(x[:,indx])  )  # auto-select range based on sampling retained
        length_scale_min_here= np.max([1e-3,0.2*np.std(x[:,indx]/np.sqrt(len(x)))])
        length_scale_bounds_est.append( (length_scale_min_here , 5*np.std(x[:,indx])   ) )  # auto-select range based on sampling *RETAINED* (i.e., passing cut).  Note that for the coordinates I usually use, it would be nonsensical to make the range in coordinate too small, as can occasionally happens

    print " GP: Input sample size ", len(x), len(y)
    print " GP: Estimated length scales "
    print length_scale_est
    print length_scale_bounds_est

        # These parameters have been hand-tuned by experience to try to set to levels comparable to typical lnL Monte Carlo error
    kernel = WhiteKernel(noise_level=0.1,noise_level_bounds=(1e-2,1))+C(0.5, (1e-3,1e1))*RBF(length_scale=length_scale_est, length_scale_bounds=length_scale_bounds_est)
    gp = GaussianProcessRegressor(kernel=kernel, n_restarts_optimizer=8)

    gp.fit(x,y)

    print  " Fit: std: ", np.std(y - gp.predict(x)),  "using number of features ", len(y) 

    if opts.fit_save_gp:
        print " Attempting to save fit ", opts.fit_save_gp
        joblib.dump(gp,opts.fit_save_gp)
        
    return lambda x: gp.predict(x)


parser = argparse.ArgumentParser()
parser.add_argument("--intermediate-file", help="Name of intermediate evaluation grid")
parser.add_argument("--integral-output",default="integral_output.dat", help="Output file for this set of parameters")
parser.add_argument("--mcz", type=float,default=None,help="Redshifted chirp mass. MUST be specified ")
parser.add_argument("--redshift",default=0,type=float, help="Redshift, assumed known ")
parser.add_argument("--pkl-file", help="Fit.  If already present, can re-use a version of the fit generated by this code")
parser.add_argument("--fit-save-gp",default=None, help="pkl file to save to. If already present, use this fit. Please use postfix .pkl")
parser.add_argument("--verbose",action='store_true')
parser.add_argument("--parameter", action='append', help="Parameters used to construct the EOS. Assume spectral parameterization for now")
parser.add_argument("--parameter-value", type=float,action='append', help="Value of parameter")
opts=  parser.parse_args()


eos_params ={}
for indx  in np.arange(len(opts.parameter)):
    param = opts.parameter[indx]
    eos_params[param] = opts.parameter_value[indx]

if not opts.intermediate_file and not opts.pkl_file:
    print " FAILURE: need input data"
    sys.exit(0)


###
### Compute source-frame chirp mass
###

if opts.mcz is None:
    print "FAILURE: Need redshifted chirp mass. (Should be able to retrieve from injection xml with detector-frame masses)"
    sys.exit(0)
z=opts.redshift
mc_source = opts.mcz/(1+z)

###
### Load grid, create fit
###

my_gp=None
if opts.fit_save_gp:
    my_gp=joblib.load(opts.pkl_file)
else:
    dat = np.loadtxt(opts.intermediate_file)  # delta LambdaTilde lnG

print "TEMPORARY HACK: Use small data set, to get code to run"
dat=dat[:500]  # temporary

X =dat[:,0:1]
Y = dat[:,-1]
my_fit = fit_gp(X,Y)

print my_fit(np.c_[[0,500]])[0]

###
### Build EOS: lambda(m) curves
###

p0_ref = 2.272499999999999930e+33
epsilon0_ref = 2.050000000000000000e+14
xmax_ref =  7.25
if not 'gamma0' in eos_params:
    eos_params['gamma0']=0.707
if not 'gamma1' in eos_params:
    eos_params['gamma1']=0.707
if not 'p0' in eos_params:
    eos_params['p0'] = p0_ref
if not 'epsilon0' in eos_params:
    eos_params['epsilon0'] = epsilon0_ref
if not 'xmax' in eos_params:
    eos_params['xmax'] = xmax_ref

print eos_params
my_eos = EOSManager.EOSLindblomSpectral(name="internal",spec_params=eos_params)
dat_mr = EOSManager.make_mr_lambda(my_eos.eos)  # r m(Msun) lambda
lam_fit = scipy.interpolate.interp1d(dat_mr[:,1], dat_mr[:,2])

#print dat_mr, lam_fit(1.4)


###
### Build lambda_tilde (eta) function
###

def lambda_tilde_here(delta):
    # compute m1,m2
    eta = 0.25*(1.0 - delta*delta)
    m1,m2 = lalsimutils.m1m2(mc_source, eta)
    lam1 = lam_fit(m1)
    lam2 = lam_fit(m2)
    lambda_tilde, dLt = lalsimutils.tidal_lambda_tilde(m1,m2,lam1,lam2)
    return lambda_tilde


###
### Construct (one-dimensional) likelihood
###

fac_standard = 16 * np.power(2,2./5.)/np.power(3-1,2)
def likelihood_function(d):  
    lam_here = lambda_tilde_here(d)
    if isinstance(d,float):
        lnG = my_fit(np.c_[[d,lam_here]])[0]
        return np.exp(lnG)* mc_source *fac_standard/np.power(1-d*d,6./5.)
    else:
        return np.exp(my_fit(np.c_[d,lam_here])) * mc_source *fac_standard/np.power(1-d*d,6./5.)


###
### Perform one-dimensional integral
###
res = scipy.integrate.quad(likelihood_function,0, 0.5)

###
### Save result
###
I = res[0]
print np.log(I), I
vals = [np.log(I), eos_params["gamma1"],eos_params["gamma2"],eos_params["gamma3"], np.log10(float(eos_params["p0"])), np.log10(float(eos_params["epsilon0"])), eos_params["xmax"]]
print vals
np.savetxt(opts.integral_output,np.array(vals).T)



#!/usr/bin/env python
#
# GOAL
#    - plot strain h_{lm}(f) per mode
#    - scale to specific masses (or use dimensionless if M=1).
#    - show SRD
#
# BONUS USES
#    - validate tapering (as needed)
#
# EXAMPLES
#     python util_EOBPlotSimulationModes.py --fmin 80 --show-plots


import argparse
import numpy as np

import RIFT.lalsimutils as lalsimutils
import lalsimulation as lalsim
import lal
import sys

import RIFT.physics.EOBTidalExternalC as eobT

parser = argparse.ArgumentParser()
parser.add_argument("--group", default="Sequence-GT-Aligned-UnequalMass",help="inspiral XML file containing injection information.")
parser.add_argument("--param", default=(0.0, 2.),help="Parameter value")
parser.add_argument("--mass1", default=1.5,type=float,help="Total mass in solar masses")
parser.add_argument("--mass2", default=1.4,type=float,help="Total mass in solar masses")
parser.add_argument("--l", default=2, type=int)
parser.add_argument("--fmin",default=30, type=float)
parser.add_argument("--verbose",action='store_true')
parser.add_argument("--incl", type=float,default=np.pi/4)
parser.add_argument("--print-group-list",default=False,action='store_true')
parser.add_argument("--print-param-list",default=False,action='store_true')
parser.add_argument("--show-plots",default=False,action='store_true')
parser.add_argument("--save-plots",default=False,action='store_true', help="Write plots to file (only useful for OSX, where interactive is default")
parser.add_argument("--seglen",default=None,type=float)
opts = parser.parse_args()



bNoInteractivePlots=True # default
fig_extension = '.jpg'
try:
    import matplotlib
    print " Matplotlib backend ", matplotlib.get_backend()
    if matplotlib.get_backend() is 'MacOSX':
        if opts.save_plots:
            print "  OSX without interactive plots"
            bNoInteractivePlots=True
            fig_extension='.jpg'
        else:  #  Interactive plots
            print "  OSX with interactive plots"
            bNoInteractivePlots=False
    elif matplotlib.get_backend() is 'agg':
        fig_extension = '.png'
        bNoInteractivePlots=True
        print " No OSX; no interactive plots "
    else:
        print " Unknown configuration "
        fig_extension = '.png'
        bNoInteractivePlots =True
    from matplotlib import pyplot as plt
    bNoPlots=False
except:
    from matplotlib import pyplot as plt
    fig_extension = '.png'
    print " - no matplotlib - "
    bNoInteractivePlots = False
    bNoPlots = True



group = opts.group
param = eval(str(opts.param))
if opts.verbose:
    print "Importing ", group, param 


if opts.print_group_list:
    print "Simulations available"
    for key in  nrwf.internal_ParametersAvailable.keys():
        print  "  ", key
    sys.exit(0)

if opts.print_param_list:
    print "Parameters available for ", group
    for key in  nrwf.internal_ParametersAvailable[group]:
        print  "  ", key
    sys.exit(0)


l = opts.l
T_window = 64
fmin =10

P=lalsimutils.ChooseWaveformParams()
P.m1 = opts.mass1 *lal.MSUN_SI
P.m2 = opts.mass2 *lal.MSUN_SI
P.dist = 150*1e6*lal.PC_SI
P.lambda1  = 500
P.lambda2  = 500
P.fmin=opts.fmin   # Just for comparison!  Obviously only good for iLIGO
P.ampO=-1  # include 'full physics'
P.deltaT=1./16384
P.taper = lalsim.SIM_INSPIRAL_TAPER_START
P.deltaF = lalsimutils.findDeltaF(P)
P.scale_to_snr(20,lalsim.SimNoisePSDaLIGOZeroDetHighPower,['H1', 'L1'])
if opts.seglen:
    P.deltaF = 1./opts.seglen
if P.deltaF > 1./T_window:
    print " time too short "
P.print_params()

wfP = eobT.WaveformModeCatalog(P,lmax=l,align_at_peak_l2_m2_emission=True)
print " Loaded modes ", wfP.waveform_modes_complex.keys()
wfP.P.incl = opts.incl

# LAL hlm(t)
hlmT_lal = lalsimutils.SphHarmTimeSeries_to_dict(lalsimutils.hlmoft(wfP.P,opts.l),opts.l)
tvals_lal = lalsimutils.evaluate_tvals(hlmT_lal[(2,2)])  #float(hlmT_lal[(2,2)].epoch) + np.arange(len(hlmT_lal[(2,2)].data.data))*hlmT_lal[(2,2)].deltaT

# EOB hlm(t)
hlmT_eob = wfP.hlmoft(force_T=1./P.deltaF)
tvals_eob = lalsimutils.evaluate_tvals(hlmT_eob[(2,2)])

###
### Time domain : (a) rescaled EOB_tidal [here], (b) LAL, and (c) rescaled NR [internal]
###
print " TIME DOMAIN PLOTS"
# Manually plot raw data
figindex =0
tmax = np.abs(wfP.waveform_modes_complex[(2,2)][-1,0])
for mode in wfP.waveform_modes_complex.keys():
 if opts.show_plots or opts.save_plots:
    figindex+=1; plt.figure(figindex)
    plt.xlim(tmax-0.1, tmax)
    plt.xlabel("$t (s)$")
    plt.ylabel("$|r/M \\tilde{h}_{lm}(t)|$")
    plt.title(" Raw data from Matlab EOB code ")
    plt.plot(wfP.waveform_modes_complex[mode][:,0], np.abs(wfP.waveform_modes_complex[mode][:,1]),'r-',label=str(mode))
    plt.plot(wfP.waveform_modes_complex[mode][:,0], np.real(wfP.waveform_modes_complex[mode][:,1]),'r--',label=str(mode))
    plt.legend()
#    plt.plot(tvals_eob,np.abs(hlmT_eob[mode].data.data), label=str(mode)+'_eob')
#    plt.plot(tvals_eob,np.real(hlmT_eob[mode].data.data), label=str(mode)+'_eob')

#    plt.figure(99);
#    if np.abs(mode[1])>0:
    figindex+=1; plt.figure(figindex)
    plt.xlabel("$t (s)$")
    plt.ylabel("$arg \\tilde{h}_{lm}(t)$")
    plt.title(" Raw data from Matlab EOB code ")
    datPhase = lalsimutils.unwind_phase(np.angle(wfP.waveform_modes_complex[mode][:,1]))    #needs to be unwound to be continuous
    plt.plot(wfP.waveform_modes_complex[mode][:,0], datPhase,label=str(mode))
    

figindex=20
for mode in hlmT_lal.keys():
  if (opts.show_plots or opts.save_plots) and mode[1]>0:
    figindex+=1
    plt.figure(figindex)
    plt.xlim(-1, 0.1)
    plt.xlabel("$t (s)$")
    plt.ylabel("$| \\tilde{h}_{lm}(t)|$")
    tvals = lalsimutils.evaluate_tvals(hlmT_lal[mode])
    plt.title(" LAL vs (interpolated,scaled) EOB Matlab results")
    plt.plot(tvals, np.abs(hlmT_lal[mode].data.data),'r-' ,label=str(mode)+"_lal")
    plt.plot(tvals, np.real(hlmT_lal[mode].data.data),'r--' ,label=str(mode)+"_lal")
    if mode in hlmT_eob.keys():
        plt.plot(tvals_eob, np.abs(hlmT_eob[mode].data.data),'b-' ,label=str(mode)+"_eob")
        plt.plot(tvals_eob, np.real(hlmT_eob[mode].data.data),'b--' ,label=str(mode)+"_eob")

    hlm_NR = wfP.waveform_modes_complex[mode]
    scaleFactorM = eobT.MsunInSec*(wfP.P.m1+wfP.P.m2)/lal.MSUN_SI
    scaleFactorDinSeconds =  wfP.P.dist/lal.C_SI
    hlm_NR[:,1] *=  scaleFactorM/scaleFactorDinSeconds  # Nominally we report r h in units of M
    plt.plot(hlm_NR[:,0], np.abs(hlm_NR[:,1]),'k', label=str(mode)+"_eob_manual")

    plt.legend()

  if opts.save_plots:
      plt.savefig("EOB_mode_"+str(mode)+fig_extension)

 
print "FREQUENCY VERSUS TIME"
if opts.show_plots or opts.save_plots:
    datPhase= lalsimutils.unwind_phase(np.angle(hlmT_eob[(2,2)].data.data))
    nStride=4
    freq = (np.roll(datPhase,nStride/2) - np.roll(datPhase,-nStride/2))/(P.deltaT*nStride)
    datPhase_lal= lalsimutils.unwind_phase(np.angle(hlmT_lal[(2,2)].data.data))
    freq_lal = (datPhase_lal - np.roll(datPhase_lal,-1))/P.deltaT
#print len(datPhase), len(freq), len(tvals_eob), len(tvals)
    plt.figure(50)
    plt.title(" Time versus frequency relationship (ESTIMATED) ")
    plt.xlabel("t(s)")
    plt.ylabel("f(Hz)")
    plt.plot(tvals_eob, freq,label='eob')
    plt.plot(tvals, freq_lal,label='lal')
    plt.plot(tvals, np.ones(len(tvals))*P.fmin) # should be tangent to starting point
    plt.ylim(0,2000)
    plt.xlim(-50,0)
    plt.legend()
    if opts.show_plots:
        plt.show()
    else:
        plt.savefig("EOB_fft.jpg")


###
### Frequency domain : (a) rescaled NR [here], (b) LAL, and (c) rescaled NR [internal]
###

# NR fourier: Via rescaling (cleaned) dimensionless FFT
plt.figure(1)
#plt.rc('text',usetex=True)
plt.xlabel("$\log_{10} f (Hz)$")
plt.ylabel('$\log_{10}|\\tilde{h}_{lm}(f)|$')
plt.ylim(-38,-20)  # astrophysical scale
plt.xlim(0.5,4)    # plausible frequency range
plt.figure(2)
plt.xlabel("$f (Hz)$")
plt.ylabel("$|\\tilde{h}_{lm}(f)|$")
plt.xlim(-200,200)  # plausible frequency range for high-mass sources
plt.figure(3)
plt.xlabel("$f (Hz)$")
plt.ylabel("$|\\tilde{h}_{lm}(f)|^2/S_h(f)$")
plt.xlim(-200,200)  # plausible frequency range for high-mass sources
plt.figure(6)

# NR fourier via hlmoff
hlmF_NR_lal = wfP.hlmoff(  deltaT=wfP.P.deltaT,force_T=1./P.deltaF)  
fvals_NR_lal = lalsimutils.evaluate_fvals(hlmF_NR_lal[(2,2)])
# lal 
hlm_lal = lalsimutils.SphHarmFrequencySeries_to_dict(lalsimutils.hlmoff(wfP.P,opts.l),opts.l)
fvals_lal = lalsimutils.evaluate_fvals(hlm_lal[(2,2)])


for mode in hlm_lal.keys():
 if opts.show_plots or opts.save_plots:
    if mode[1]>0 and mode in hlmF_NR_lal.keys():
        print " Handling mode ", mode
        # Re-evaluate the frequency sampling each time
        fvals_lal =lalsimutils.evaluate_fvals(hlm_lal[mode])
        fvals_NR_lal =lalsimutils.evaluate_fvals(hlmF_NR_lal[mode])
        print mode, len(fvals_lal), len(fvals_NR_lal)


        plt.figure(1); plt.xlim(1, 4); 
        plt.plot(np.log10(fvals_lal), np.log10(np.abs(hlm_lal[mode].data.data)),eobT.mode_line_style[mode], label=str(mode)+"_lal")
        plt.plot(np.log10(fvals_NR_lal), np.log10(np.abs(hlmF_NR_lal[mode].data.data)),eobT.mode_line_style[mode], label=str(mode)+"_eob",lw=2) 
        plt.figure(2); plt.xlim(20, 200);
        plt.plot(fvals_lal, np.abs(hlm_lal[mode].data.data),eobT.mode_line_style[mode], label=str(mode)+"_lal")
        plt.plot(fvals_NR_lal, np.abs(hlmF_NR_lal[mode].data.data),eobT.mode_line_style[mode], label=str(mode)+"_eob",lw=2)
        plt.figure(3); plt.xlim(20, 200);
        datSh_lal = map(lalsim.SimNoisePSDaLIGOZeroDetHighPower, np.abs(fvals_lal))
        datSh_eob = map(lalsim.SimNoisePSDaLIGOZeroDetHighPower, np.abs(fvals_NR_lal))
        plt.plot(fvals_lal, np.abs(hlm_lal[mode].data.data)**2/datSh_lal,eobT.mode_line_style[mode], label=str(mode)+"_lal")
        plt.plot(fvals_NR_lal, np.abs(hlmF_NR_lal[mode].data.data)**2/datSh_eob,eobT.mode_line_style[mode], label=str(mode)+"_eob",lw=2)


if opts.show_plots or opts.save_plots:
    plt.figure(1);plt.legend()
    plt.figure(2);plt.legend()
    plt.figure(3);plt.legend()
    if opts.show_plots:
        plt.show()
    else:
        plt.figure(1);  plt.savefig("EOB_plt1"+fig_extension)
        plt.figure(2);  plt.savefig("EOB_plt2"+fig_extension)
        plt.figure(3);  plt.savefig("EOB_plt3"+fig_extension)


###
### Inner products
###

print hlm_lal[(2,2)].deltaF, len(hlm_lal[(2,2)].data.data)
print hlmF_NR_lal[(2,2)].deltaF, len(hlmF_NR_lal[(2,2)].data.data)
IP = lalsimutils.CreateCompatibleComplexOverlap(hlm_lal)

for mode in hlm_lal:
  if mode in hlmF_NR_lal.keys():
    print mode, IP.norm(hlm_lal[mode]), IP.norm(hlmF_NR_lal[mode]), IP.ip(hlm_lal[mode],hlmF_NR_lal[mode])/(IP.norm(hlm_lal[mode]) * IP.norm(hlmF_NR_lal[mode]))



#! /usr/bin/env python
#  
#
# Tool: dumps a candidate waveform to a frame file.
# Default GPS time used is boring
# Tries to be as close to C as possible -- no interface via pylal/glue
#
# EXAMPLE
#   python util_NRWriteFrame.py --group 'Sequence-SXS-All' --param 1 --verbose
#   python util_NRWriteFrame.py --incl 1.5 --verbose     # edge on
#
#  WARNING: My version does NOT interpolate the signal to a synchronized set of sample times.
#                       This may cause problems for some applications, particularly at low sample rates.


import argparse
import sys
import numpy as np
import RIFT.lalsimutils as lalsimutils
import lalsimulation as lalsim
import lalframe
import lal

import RIFT.physics.EOBTidalExternalC as eobwf


parser = argparse.ArgumentParser()
parser.add_argument("--fname", default=None, help = "Base name for output frame file. Otherwise auto-generated ")
parser.add_argument("--instrument", default="H1",help="Use H1, L1,V1")
parser.add_argument("--inj", dest='inj', default=None,help="inspiral XML file containing injection information. Used for extrinsic information only")
parser.add_argument("--mass1", default=1.50,type=float,help="Mass in solar masses")  # 150 turns out to be ok for Healy et al sims
parser.add_argument("--mass2", default=1.35,type=float,help="Mass in solar masses")
parser.add_argument("--lambda1",default=590,type=float)
parser.add_argument("--lambda2", default=590,type=float)
parser.add_argument("--fmin", default=35,type=float,help="Mininmum frequency in Hz, default is 40Hz to make short enough waveforms. Focus will be iLIGO to keep comutations short")
parser.add_argument("--lmax", default=2, type=int)
parser.add_argument("--event",type=int, dest="event_id", default=None,help="event ID of injection XML to use.")
parser.add_argument("--srate",type=int,default=16384,help="Sampling rate")
parser.add_argument("--seglen", type=float,default=256*2., help="Default window size for processing.")
parser.add_argument("--incl",default=0,type=float,help="Set the inclination of the simuation. Helpful for aligned spin tests")
parser.add_argument("--start", type=int,default=None)
parser.add_argument("--stop", type=int,default=None)
parser.add_argument("--approx",type=str,default=None,help="Unused")
parser.add_argument("--single-ifo",action='store_true',default=None,help="Unused")
parser.add_argument("--verbose", action="store_true",default=False, help="Required to build post-frame-generating sanity-test plots")
parser.add_argument("--save-plots",default=False,action='store_true', help="Write plots to file (only useful for OSX, where interactive is default")
opts=  parser.parse_args()



# Window size : 8 s is usually more than enough, though we will fill a 16s buffer to be sure.
T_window = 128. # default.  Note in practice most NS-NS waveforms will need to be many tens of minutes long 



# Generate signal
P=lalsimutils.ChooseWaveformParams()
P.m1 = opts.mass1 *lal.MSUN_SI
P.m2 = opts.mass2 *lal.MSUN_SI
P.dist = 150*1e6*lal.PC_SI
P.lambda1  = opts.lambda1
P.lambda2  = opts.lambda2
P.fmin=opts.fmin   # Just for comparison!  Obviously only good for iLIGO
P.ampO=-1  # include 'full physics'
P.deltaT=1./16384
P.taper = lalsim.SIM_INSPIRAL_TAPER_START
# This must be done BEFORE changing the duration
P.scale_to_snr(20,lalsim.SimNoisePSDaLIGOZeroDetHighPower,['H1', 'L1'])
if opts.start and opts.stop:
    opts.seglen = opts.stop-opts.start # override
P.deltaF = 1./opts.seglen #lalsimutils.findDeltaF(P)
if P.deltaF > 1./T_window:
    print " time too short "


if not opts.inj:
    P.taper = lalsimutils.lsu_TAPER_START
    P.tref = 1000000000 #1000000000            # default
    P.dist = 150*1e6*lal.PC_SI # default
else:
    from glue.ligolw import lsctables, table, utils # check all are needed

    filename = opts.inj
    event = opts.event_id
    xmldoc = utils.load_filename(filename, verbose = True,contenthandler =lalsimutils.cthdler)
    sim_inspiral_table = table.get_table(xmldoc, lsctables.SimInspiralTable.tableName)
    P.copy_sim_inspiral(sim_inspiral_table[int(event)])
P.detector = opts.instrument

P.print_params()

# FAIL if masses are not viable
if P.m1/lal.MSUN_SI > 3 or P.m2/lal.MSUN_SI > 3:
    print " Invalid NS mass "
    sys.exit(0)

wfP = eobwf.WaveformModeCatalog(P,lmax=opts.lmax)
print " Loaded modes ", wfP.waveform_modes_complex.keys()
print " Duration of stored signal ", wfP.estimateDurationSec()
mtotMsun = (wfP.P.m1+wfP.P.m2)/lal.MSUN_SI


# Generate signal
hoft = wfP.real_hoft()   # include translation of source, but NOT interpolation onto regular time grid
print " Original signal (min, max) ", np.min(hoft.data.data) ,np.max(hoft.data.data)
print " Original signal duration ", hoft.deltaT*hoft.data.length
# zero pad to be opts.seglen long
TDlenGoal = int(opts.seglen/hoft.deltaT)
if TDlenGoal < hoft.data.length:
    print " seglen too short -- signal truncation would be required"
    sys.exit(0)
nptsOrig = hoft.data.length
hoft = lal.ResizeREAL8TimeSeries(hoft, 0, TDlenGoal)
hoft.data.data[nptsOrig:TDlenGoal] = 0 # np.zeros(TDlenGoal-nptsOrig) # zero out the tail
print " Resized signal (min, max) ", np.min(hoft.data.data) ,np.max(hoft.data.data)
# zero pad some more on either side, to make sure the segment covers start to stop
if opts.start and hoft.epoch > opts.start:
    nToAddBefore = int((hoft.epoch-opts.start)/hoft.deltaT)
    print "Padding start ", nToAddBefore, hoft.data.length
    ht = lal.CreateREAL8TimeSeries("Template h(t)", 
            hoft.epoch - nToAddBefore*hoft.deltaT, 0, hoft.deltaT, lalsimutils.lsu_DimensionlessUnit, 
            hoft.data.length+nToAddBefore)
    ht.data.data[0:ht.data.length] = np.zeros(ht.data.length)  # initialize to zero for safety
    ht.data.data[nToAddBefore:nToAddBefore+hoft.data.length] = hoft.data.data
    hoft = ht

if opts.stop and hoft.epoch+hoft.data.length*hoft.deltaT < opts.stop:
    nToAddAtEnd = int( (-(hoft.epoch+hoft.data.length*hoft.deltaT)+opts.stop)/hoft.deltaT)
else:
    nToAddAtEnd=0
if nToAddAtEnd <=0:
    nToAddAtEnd = int(1/hoft.deltaT)  # always at at least 1s of padding at end
print "Padding end ", nToAddAtEnd, hoft.data.length
nptsNow = hoft.data.length
hoft = lal.ResizeREAL8TimeSeries(hoft,0, int(hoft.data.length+nToAddAtEnd))
hoft.data.data[nptsNow:hoft.data.length] = 0
print " Padded signal (min, max) ", np.min(hoft.data.data) ,np.max(hoft.data.data)


channel = opts.instrument+":FAKE-STRAIN"

tstart = int(hoft.epoch)
duration = int(hoft.data.length*hoft.deltaT)
if not opts.fname:
    fname = opts.instrument.replace("1","")+"-fake_strain-"+str(tstart)+"-"+str(duration)+".gwf"

print "Writing signal with ", hoft.data.length*hoft.deltaT, " to file ", fname
print "Maximum original ", np.max(hoft.data.data)
print "Start time", hoft.epoch
lalsimutils.hoft_to_frame_data(fname,channel,hoft)

bNoInteractivePlots=True # default
fig_extension = '.jpg'
try:
    import matplotlib
    print " Matplotlib backend ", matplotlib.get_backend()
    if matplotlib.get_backend() is 'MacOSX':
        if opts.save_plots:
            print "  OSX without interactive plots"
            bNoInteractivePlots=True
            fig_extension='.jpg'
        else:  #  Interactive plots
            print "  OSX with interactive plots"
            bNoInteractivePlots=False
    elif matplotlib.get_backend() is 'agg':
        fig_extension = '.png'
        bNoInteractivePlots=True
        print " No OSX; no interactive plots "
    else:
        print " Unknown configuration "
        fig_extension = '.png'
        bNoInteractivePlots =True
    from matplotlib import pyplot as plt
    bNoPlots=False
except:
    from matplotlib import pyplot as plt
    fig_extension = '.png'
    print " - no matplotlib - "
    bNoInteractivePlots = True
    bNoPlots = False


# TEST: Confirm it works by reading the frame
if opts.verbose and not bNoPlots:
    import os
#    from matplotlib import pyplot as plt
    # First must create corresponding cache file
    os.system("echo "+ fname+ " | lalapps_path2cache   > test.cache")
    # Now I can read it
    # Beware that the results are OFFSET FROM ONE ANOTHER due to PADDING,
    #    but that the time associations are correct
    hoft2 = lalsimutils.frame_data_to_hoft("test.cache", channel)
    tvals2 = (float(hoft2.epoch) - float(wfP.P.tref)) +  np.arange(hoft2.data.length)*hoft2.deltaT
    tvals = (float(hoft.epoch) - float(wfP.P.tref)) +  np.arange(hoft.data.length)*hoft.deltaT

    ncrit = np.argmax(hoft2.data.data)
    tcrit = float(hoft2.epoch) - float(wfP.P.tref) + ncrit*hoft2.deltaT    # zero time

    print " Maximum original ", np.max(hoft.data.data), " size ", len(tvals), len(hoft.data.data)
    print " Maximum frames ", np.max(hoft2.data.data), " size ", len(tvals2), len(hoft2.data.data)
    print " Location of maximum in samples. relative time ", ncrit, tcrit
    print " Location of maximum in samples, compared to tref", tcrit+P.tref, 
    print " Location of maximum as GPS time ", ncrit*hoft2.deltaT+ float(hof2.epoch)

    plt.plot(tvals2,hoft2.data.data,label='Fr')
    plt.xlim(tcrit-1,tcrit+1)
    plt.plot(tvals,hoft.data.data,label='orig')
    plt.legend(); 

    if not bNoInteractivePlots:
        plt.show()
    else:
        for indx in [1]:
            print "Writing figure ", indx
            plt.xlim(tcrit-0.1,tcrit+0.01)
            plt.figure(indx); plt.savefig("eob-framedump-" +str(indx)+fig_extension)
#            plt.xlim(min(tvals2),max(tvals2)) # full range with pad
#            plt.figure(indx); plt.savefig("eob-framedump-full-" +str(indx)+fig_extension)





#! /usr/bin/env python
#
#
# EXAMPLE
#    python util_NRNearestSimulationTo.py --approx SEOBNRv2 --nr-group Sequence-RIT-Generic  --verbose --srate 4096 --fname overlap-grid.xml.gz  --force-aligned-interpret
#    python util_NRNearestSimulationTo.py --fname-fisher fisher_gamma.dat --fname overlap-grid.xml.gz  --npts 3 --approx SEOBNRv2  --npts 3 
#
# WHERE TO GET FISHER MATRICIES
#   + Match calculations
# 
#     python util_ManualOverlapGrid.py --inj mdc.xml.gz --event 0 --parameter mtot --parameter-range '[50,70]' --use-fisher --seglen 8 --verbose --reset-grid-via-match
#     python util_ManualOverlapGrid.py --inj mdc.xml.gz --event 0 --parameter mtot --parameter-range '[50,70]' --use-fisher --seglen 8 --verbose --reset-grid-via-match --parameter eta --parameter-range '[0.1,0.2499]'
#
#   + Real overlap

import argparse
import sys
import numpy as np
import scipy
import RIFT.lalsimutils as lalsimutils
import lalsimulation as lalsim
import lalframe
import lal
import functools
import itertools

from scipy.optimize import brentq



import NRWaveformCatalogManager as nrwf
import scipy.optimize as optimize

parser = argparse.ArgumentParser()
# Parameters
parser.add_argument("--cut",default=0.001,type=float,help="Reject points if nothing is closer than this")
parser.add_argument('--nr-group', default=None,help="NR group to search (otherwise, will loop over all)")
parser.add_argument("--force-aligned-interpret",default=False,action='store_true')
parser.add_argument('--fname-fisher', default=None,help="Fisher name")
parser.add_argument('--fname', default=None,help="Name for XML file")
parser.add_argument('--fname-output', default="suggested_placement_from_posterior",help="Name for XML file for output")
parser.add_argument('--npts', default=None,type=int,help="Number of elements of the XML file to use (truncation)")
parser.add_argument("--approx",type=str,default=None,help="If supplied, the overlaps are done using this approximant, instead of a Fisher matrix")
parser.add_argument("--fisher-psd",type=str,default="SimNoisePSDaLIGOZeroDetHighPower",help="psd name (attribute in lalsimulation).  SimNoisePSDiLIGOSRD, lalsim.SimNoisePSDaLIGOZeroDetHighPower, lalsimutils.Wrapper_AdvLIGOPsd, .SimNoisePSDiLIGOSRD... ")
parser.add_argument("--psd-file",default=None,help="PSD file (assumed for hanford)")
parser.add_argument("--match-tolerance",default=3e-3,type=float)
parser.add_argument("--srate",type=int,default=16384,help="Sampling rate")
parser.add_argument("--seglen", type=int,default=16., help="Default window size for processing.")
parser.add_argument("--fmin", default=20,type=float,help="Mininmum frequency in Hz, default is 20Hz.")
parser.add_argument("--fmax", default=1700,type=float,help="Mininmum frequency in Hz, default is 20Hz.")
parser.add_argument("--verbose", action="store_true",default=False, help="Spam")
opts=  parser.parse_args()

T_window = int(opts.seglen)
srate = opts.srate
deltaT = 1./srate
###
### Key routine
###

def nr_closest_to(P,distance_function_P1P2,nr_group,mass_ref=None,return_list=False):


    mass_start_msun = 70
    if not(mass_ref is None):
        mass_start_msun = mass_ref
    result_list = []
    for param  in nrwf.internal_ParametersAvailable[nr_group]:
        acat = nrwf.WaveformModeCatalog(nr_group,param,metadata_only=True)
        P1 = acat.P
        P1.deltaF = P.deltaF
        P1.deltaT = P.deltaT
        P1.fmin =P.fmin= opts.fmin
        if opts.force_aligned_interpret:
            P.s1x = P.s1y =0
            P.s2x = P.s2y=0
            P1.s1x = P1.s1y =0
            P1.s2x = P1.s2y=0
        # print " ---> preparing for loop "
        # P.print_params()
        # P1.print_params()
        def distance_at_mass(m_msun):
            if m_msun < 20:
                return 1e5
            if opts.verbose:
                print " Trying ", nr_group, param, m_msun
            P1.assign_param('mtot', m_msun*lal.MSUN_SI)
            return distance_function_P1P2(P,P1)
        
        res = optimize.minimize(distance_at_mass, mass_start_msun,bounds=[(20,200)],tol=opts.match_tolerance,method='Nelder-Mead',options={'maxiter':15}) # loose tolerance because quadratic, often noisy calculations due to fmin, etc
        if opts.verbose:
            print " ===> search result <=== "
            P.print_params(); print "  ", nr_group, param,  res.x[0]
        val = distance_at_mass(res.x[0])
        result_list.append( (param,res.x[0],val))
        mass_start_msun = res.x  # use guess for next point, to speed up things
        
    # Full output: param, M, value for the whole group
    if opts.verbose:
        print result_list
    if return_list:
        return result_list

    # short output
    
    xvals = np.array([x for a,b,x in result_list])
    indx = np.argmin(xvals)
    return [result_list[indx][0],result_list[indx][-1]]
    


def make_distance_for_fisher(mtx, param_names):
    def my_distance(P1,P2):
        # extract parameters for P1
        vec1 = np.zeros(len(param_names))
        for indx in np.arange(len(param_names)):
            vec1[indx] = P1.extract_param(param_names[indx])
            if param_names[indx] in ['mc', 'mtot','m1', 'm2']:
                vec1[indx] /= lal.MSUN_SI

        # extract parameters for P2
        vec2 = np.zeros(len(param_names))
        for indx in np.arange(len(param_names)):
            vec2[indx] = P2.extract_param(param_names[indx])
            if param_names[indx] in ['mc', 'mtot','m1', 'm2']:
                vec2[indx] /= lal.MSUN_SI


        deltaV = vec1-vec2
        return np.dot(deltaV, np.dot(mtx,deltaV))

    return my_distance


dist_here = None
if opts.fname_fisher:
    ###
    ### Load in Fisher. Create default distance function (WARNING: needs parameter labels to be appended!)
    ###
    datFisher = np.genfromtxt(opts.fname_fisher,names=True)
    print " Fisher matrix ", datFisher, " with shape ",  datFisher.shape , " and parameters ", datFisher.dtype.names

    # parameter names
    param_names = list(datFisher.dtype.names)

    # FAIL if Fisher matrix does not include a mass-like dimension
    if not ( 'mc' in param_names  or 'mtot' in param_names or 'm1' in param_names):
        print " You must have a mass parameter in your Fisher matrix, otherwise you cannot optimize in mass "
        sys.exit(0)

    # parameter matrix
    mtx =[]
    if len(param_names) > 1:
        mtx = np.zeros( (len(param_names),len(param_names)) )
        for indx1 in np.arange(len(param_names)):
            for indx2 in np.arange(len(param_names)):
                mtx = datFisher[indx1][indx2]
    else:
        mtx = np.zeros( (1,1))
        mtx[0,0] = datFisher[param_names[0]] #datFisher[0]

    print " Fisher in matrix form is ", mtx

    # make distance function
    dist_here = make_distance_for_fisher(mtx,param_names)
elif opts.approx:

    if not opts.psd_file:
    #eff_fisher_psd = eval(opts.fisher_psd)
        eff_fisher_psd = getattr(lalsim, opts.fisher_psd)   # --fisher-psd SimNoisePSDaLIGOZeroDetHighPower   now
        analyticPSD_Q=True
    else:
        print " Importing PSD file ", opts.psd_file
        eff_fisher_psd = lalsimutils.load_resample_and_clean_psd(opts.psd_file, 'H1', 1./opts.seglen)
        analyticPSD_Q = False

    ###
    ###  Create the  inner product function, etc needed (distance =match)
    ###
    P=lalsimutils.ChooseWaveformParams()
    P.m1 = P.m2 = 50*lal.MSUN_SI
    P.approx = lalsim.GetApproximantFromString(opts.approx)
    P.deltaT = 1./srate
    P.deltaF = 1./opts.seglen
    hfBase = lalsimutils.complex_hoff(P)
    IP = lalsimutils.CreateCompatibleComplexOverlap(hfBase,analyticPSD_Q=analyticPSD_Q,psd=eff_fisher_psd,fMax=opts.fmax,interpolate_max=True)
    def my_distance(P1,P2):
        global IP
        global opts
        P1.approx = P2.approx = lalsim.GetApproximantFromString(opts.approx);
        P1.fmin = P2.fmin = opts.fmin
        P1.deltaF = P2.deltaF = 1./T_window

        # if opts.verbose:
        #     print " ---> Inside distance function < "
        #     P1.print_params()
        #     P2.print_params()
        dist = 1e5
        try:
            hF1 = lalsimutils.complex_hoff(P1)
            hF2 = lalsimutils.complex_hoff(P2)
            rho1 = IP.norm(hF1)
            rho2 = IP.norm(hF2)
            dist = 1- np.abs( IP.ip(hF1,hF2)/rho1/rho2)
        except:
            print " Distance evaluation failure "
        if np.isnan(dist):
            return 1e5
        return dist

    dist_here = my_distance

    # print " --->  Testing <--- "
    # print my_distance(P,P)


    # print " --->  Testing <--- "
    # P2 = P.manual_copy()
    # P2.assign_param('mtot', 70*lal.MSUN_SI)
    # print my_distance(P,P2)


###
### Load in xml
###
if not opts.fname:
    print " No data provided "
    sys.exit(0)
P_list = lalsimutils.xml_to_ChooseWaveformParams_array(opts.fname)
if not (opts.npts==None) and opts.npts>0 and opts.npts < len(P_list):
    print " Truncating list of sample points to examine to size ", opts.npts
    P_list = P_list[:opts.npts]


###
### Loop over XML
###
if opts.nr_group:
    glist = [opts.nr_group]
else:
    glist = nrwf.internal_ParametersAvailable.keys()
best_fits =[]
single_best_fits = []
P_list_retain = []
dist_list_retain =[]
for P in P_list:
    if P.m1/lal.MSUN_SI < 1 or P.m2 < lal.MSUN_SI:
        # Skip crazy things that cause problems
        continue
    P.fmin = opts.fmin # we change this. FOR PRECESSING SYSTEMS THAT CAN CHANGE THE MEANING, BE CAREFUL
    if opts.verbose:
        print " Trying next point in XML"
        P.print_params()
    P.approx = lalsim.GetApproximantFromString(opts.approx)
    P.fmin = opts.fmin
    P.deltaT = 1./srate
    P.deltaF = 1./T_window

    # Loop over everything, and find best point
    best_fits_here = []
    closest_so_far_d = 1e10
    closest_so_far = ()
    for group in glist:
#        if opts.verbose:
#            print " Checking ", group
        res = nr_closest_to(P, dist_here,group)  # pair of values returned
        if res[1] < closest_so_far_d:
            closest_so_far  = (group,res[0])
            closest_so_far_d = res[1]
            if opts.verbose:
                print " NEW CLOSEST SIMULATION FOR THIS POSTERIOR POINT", closest_so_far, closest_so_far_d

    if closest_so_far_d> opts.cut:
        P_list_retain.append(P)
        dist_list_retain.append(closest_so_far_d)
    else:
        print " Skipping point shown below as too close to simulations because the closest has distance", closest_so_far_d, " which is less than ", opts.cut
        P.print_params()
        

lalsimutils.ChooseWaveformParams_array_to_xml(P_list_retain, fname=opts.fname_output, fref=P.fref)
np.savetxt(opts.fname_output+"_distances.dat", np.array([dist_list_retain]).T)




#! /usr/bin/env python
#
# GOALS
#   - takes top level directory name for BasicIteration DAG workflow
#   - uses ppc-args.txt to perform a set of plots for those iterations
#
# EXAMPLE
#   ./driver_PlotAllIterations.py --working-directory analyze_0_SEOBNRv3/0-SEOBNRv3-fmin20-Lmax2-Iterative-v0_cit --ppc-args ppc-args.txt



import argparse
import sys
import os
import shutil
import numpy as np
import lalsimutils
import lalsimulation as lalsim
import lal
import functools
import itertools

from glue import pipeline # https://github.com/lscsoft/lalsuite-archive/blob/5a47239a877032e93b1ca34445640360d6c3c990/glue/glue/pipeline.py

import dag_utils
from dag_utils import mkdir
from dag_utils import which

parser = argparse.ArgumentParser()
parser.add_argument("--working-directory",default="./")
parser.add_argument("--ppc-args",default=None,help="filename of args_ppc.txt file  which holds PPC arguments.  Should NOT contain --posterior-file , label, or line styles: these will be auto-set by the script")
opts=  parser.parse_args()

with open(opts.ppc_args) as f:
    ppcargs_list = f.readlines()
ppcargs = ' '.join( map( lambda x: x.replace('\\',''),ppcargs_list) )
ppcargs = ' '.join(ppcargs.split(' ')[1:])
# Some argument protection for later
ppcargs = ppcargs.replace('[', ' \'[')
ppcargs = ppcargs.replace(']', ']\'')
ppcargs=ppcargs.rstrip()
print "CIP", ppcargs


os.chdir(opts.working_directory)

exe = dag_utils.which("plot_posterior_corner.py")
cvt = dag_utils.which("convert_output_format_ile2inference")




cmd = exe + ' ' + ppcargs
cmd += ' --use-legend '
import glob
fnames = glob.glob("overlap-grid-*.xml.gz")
indx_names = map( lambda x: int(x.replace('overlap-grid-','').replace('.xml.gz','')),fnames)
for indx in indx_names:
    fname_xml = "overlap-grid-"+str(indx)+".xml.gz"
    fname_post = "posterior_samples-"+str(indx)+".dat"
    os.system(cvt + " "+fname_xml + " > " + fname_post) 
    new_args = ' --posterior-file ' + fname_post + ' --posterior-label ' + str(indx)
    cmd += new_args

fname_comp= glob.glob("all.net")
if len(fname_comp) >0:
    cmd+= ' --composite-file all.net '

print cmd
os.system(cmd)




#! /usr/bin/env python


import argparse

import lal
import numpy as np
import RIFT.lalsimutils as lalsimutils

import NRWaveformCatalogManager as nrwf

###
### Get hdot
###

parser = argparse.ArgumentParser()
# Parameters
parser.add_argument("--group", default="Sequence-GT-Aligned-UnequalMass",help="Specific NR simulation group to use")
parser.add_argument("--param", default=(0.0,2.),help="Parameter value")
parser.add_argument("--lmax",default=2,type=int)
opts=  parser.parse_args()



wfP = nrwf.WaveformModeCatalog(opts.group, opts.param, clean_initial_transient=True,clean_final_decay=True, shift_by_extraction_radius=True, lmax=opts.lmax,align_at_peak_l2_m2_emission=True,build_strain_and_conserve_memory=True)
wfP.P.dist = 1e6*lal.PC_SI  # avoid underflow
hlmT = wfP.hlmoft()
hlmT_dot = wfP.hlmoft()  # allocate fresh copy to hold derivatives
hlmT_memory_integrand = wfP.hlmoft()  # allocate another copy to hold results
hlmT_memory = wfP.hlmoft()  # allocate another copy to hold results
for pair in hlmT:
    dt = hlmT[pair].deltaT
    dat = hlmT[pair].data.data
    hlmT_dot[pair].data.data = (dat - np.roll(dat,-1))/dt
    hlmT_memory_integrand[pair].data.data *=0  # zero out 
    hlmT_memory[pair].data.data *=0  # zero out 

# Expression 0: Aligned spin (being lazy: Eq. 2.8 in 1108.3121
# Should find python code for Clebsh-Gordon coefficients
R_in_sec  = wfP.P.dist / lal.C_SI
hlmT_memory_integrand[(2,0)].data.data = (R_in_sec)* np.sqrt(15./(2*np.pi))/42 * ( 2* np.power(np.abs(hlmT_dot[(2,2)].data.data),2)  - 2*np.power(np.abs(hlmT_dot[(2,0)].data.data),2))
hlmT_memory_integrand[(2,2)].data.data = (R_in_sec)* np.sqrt(15./(2*np.pi))/21 * hlmT_dot[(2,2)].data.data  * hlmT_dot[(2,0)].data.data
hlmT_memory_integrand[(2,-2)].data.data = (R_in_sec)* np.sqrt(15./(2*np.pi))/21 * hlmT_dot[(2,-2)].data.data  * hlmT_dot[(2,0)].data.data
if (4,4) in hlmT.keys():
    hlmT_memory_integrand[(4,0)].data.data = (R_in_sec)* np.sqrt(5./(2*np.pi))/1260 * (  np.power(np.abs(hlmT_dot[(2,2)].data.data),2)  +3*np.power(np.abs(hlmT_dot[(2,0)].data.data),2))
    hlmT_memory_integrand[(4,2)].data.data = (R_in_sec)* np.sqrt(3./(2*np.pi))/252 * hlmT_dot[(2,2)].data.data  * hlmT_dot[(2,0)].data.data
    hlmT_memory_integrand[(4,-2)].data.data = (R_in_sec)* np.sqrt(3./(2*np.pi))/252 * hlmT_dot[(2,-2)].data.data  * hlmT_dot[(2,0)].data.data
    hlmT_memory_integrand[(4,4)].data.data = (R_in_sec)* np.sqrt(14./(2*np.pi))/504 * np.power(hlmT_dot[(2,2)].data.data ,2)
    hlmT_memory_integrand[(4,-4)].data.data = (R_in_sec)* np.sqrt(14./(2*np.pi))/504 * np.power(hlmT_dot[(2,-2)].data.data ,2)

# Integrate term by term
for pair in hlmT_memory:
    dat =     hlmT_memory_integrand[pair].data.data
    dt = hlmT_memory_integrand[pair].deltaT
    hlmT_memory[pair].data.data = np.cumsum(dat)*dt

    print pair, np.max(hlmT_memory[pair].data.data), np.max(np.abs(hlmT[pair].data.data)), np.max(np.abs(hlmT_dot[pair].data.data))


# Save
for pair in hlmT_memory:
    tvals = float(hlmT_memory[pair].epoch) + hlmT_memory[pair].deltaT*np.arange(hlmT_memory[pair].data.length)#lalsimutils.evalate_tvals(hlmT_memory[pair])
    np.savetxt("memory_"+str(pair).replace(' ','')+".dat", np.array([tvals, np.real(hlmT_memory[pair].data.data)]).T)
    np.savetxt("h_"+str(pair).replace(' ','')+".dat", np.array([tvals, np.real(hlmT[pair].data.data),np.imag(hlmT[pair].data.data)]).T)



#! /usr/bin/env python
"""
  xmlInspiralToILEPinned.py
   - Script to read an injction xml and generate ILE-style pinned parameter arguments, as desired
    ./xmlInspiralToILEPinned.py mdc.xml.gz 0 polarization inclination  phi_orb distance right_ascension declination t_ref
   - Intended for use in makefiles and command lines
   - Can provide any subset of parameters as needed
"""
import sys
#from lalsimutils import *

# https://www.lsc-group.phys.uwm.edu/daswg/projects/glue/doc/glue.ligolw-module.html
from glue.ligolw import table
from glue.ligolw import lsctables
from glue.ligolw import utils

import numpy as np

def mc(m1, m2):
    return  np.power(m1 * m2, (3./5.))/np.power(m1+m2,1./5.)

filename = sys.argv[1]
event = sys.argv[2]
params = sys.argv[2::]
xmldoc = utils.load_filename(filename, verbose = True)

sim_inspiral_table = table.get_table(xmldoc, lsctables.SimInspiralTable.tableName)
row = sim_inspiral_table[int(event)]

if "right_ascension" in params:
    print " --right-ascension ", row.longitude,

if "declination" in params:
    print " --declination ", row.latitude,

if "inclination" in params:
    print " --inclination ", row.inclination,

if "polarization" in params:
    print " --psi ", row.polarization,


if "phi_orb" in params:
    print " --phi-orb ", row.coa_phase,

import lal
if "distance" in params:
    print " --distance ", row.distance # *lal.LAL_PC_SI*1e6,   # currently, idiotically

if "t_ref" in params:
    print " --time ", row.geocent_end_time + 1.0e-9*row.geocent_end_time_ns

#print " --iota", row.inclination, " --psi", row.polarization , " --dec", row.latitude, " --ra", row.longitude, " --dist", row.distance, " --phi", row.coa_phase



#
"""
ourio.py : 
  Input-output routines
   - plot sampling distributions, priors
   - plot (weighted) histograms of results
"""

import numpy as np
try:
    from matplotlib import pylab as plt   # if this fails, don't do it.  Will let us run on the cluster
except:
    print "No matplotlib for you!"
import lal   # rescale distance units if needed

"""
dumpSamplesToFile
Dump output in a format consistent with the bayesian MCMC plotting infrastructure
              This will make uptake and comparison much easier.
FORMAT:
    <key-line>
    <data-line>
where <key-line> must include (in any order)
    indx   logl  m1 m2 ra dec incl phiref psi d(Mpc) t
though in special cases m1, m2 may not be provided.

It is expected that the output will only contain a subset.
The interpretation of our result depends on how many binaries this sample.
For simplicity we will assume samples are distributed througout
"""
def dumpSamplesToFile(fname, samps, labels):
    headline = ' '.join(labels)
    np.savetxt(fname, samps,header=headline)
#    np.savetxt(fname, samps)  # stupid, but needed for cluster work.  No more!
    return 0


"""
dumpStatisticsToFile
Dump the maximum likelihood point and 1d width from a set of samples.
This will be used principally to *seed subsequent MC* with good choices, so we converge faster.
[Remember, we are free to choose a sampling distribution for each intrinsic evaluation.]
The routines using this process should take care not to overconverge.
"""
def dumpStatisticsToFile(fname,samps,labels):
    indxmax = np.argmax(samps,-1)
    pairs = [(labels[i], samps[indxmax][i]) for i in np.arange(len(labels))]
    np.savetxt(fname,pairs)
    return 0


def readStatisticsFromFile(fname,samps,labels):
    return 0



# Plotting routines
def saveParameterDistributions(fnameBase, sampler, samplerPrior):
    nFig = 0
    for param in sampler.params:
        nFig+=1
        plt.figure(nFig)
        plt.clf()
        xLow = sampler.llim[param]
        xHigh = sampler.rlim[param]
        xvals = np.linspace(xLow,xHigh,500)
        pdfPrior = samplerPrior.pdf[param]
        pdfvalsPrior = pdfPrior(xvals)/samplerPrior._pdf_norm[param]  # assume vectorized
        pdf = sampler.pdf[param]
        cdf = sampler.cdf[param]
        pdfvals = pdf(xvals)/sampler._pdf_norm[param]
        cdfvals = cdf(xvals)
        if str(param) ==  "dist":
            xvvals = xvals/(1e6*lal.LAL_PC_SI)       # plot in Mpc, not m.  Note PDF has to change
            pdfvalsPrior = pdfvalsPrior * (1e6*lal.LAL_PC_SI) # rescale units
            pdfvals = pdfvals * (1e6*lal.LAL_PC_SI) # rescale units
        plt.plot(xvals,pdfvalsPrior,label="prior:"+str(param),linestyle='--')
        plt.plot(xvals,pdfvals,label=str(param))
        plt.plot(xvals,cdfvals,label='cdf:'+str(param))
        plt.xlabel(str(param))
        plt.legend()
        plt.savefig(fnameBase+str(param)+".pdf")

def plotParameterDistributions(titleBase, sampler):
    nFig = 0
    for param in sampler.params:
        nFig+=1
        plt.figure(nFig)
        plt.clf()
        xLow = sampler.llim[param]
        xHigh = sampler.rlim[param]
        xvals = np.linspace(xLow,xHigh,500)
        pdfPrior = sampler.prior_pdf[param]
        pdfvalsPrior = pdfPrior(xvals)
        pdf = sampler.pdf[param]
        cdf = sampler.cdf[param]
        pdfvals = pdf(xvals)/sampler._pdf_norm[param]
        cdfvals = cdf(xvals)
        if str(param) ==  "dist":
            xvvals = xvals/(1e6*lal.LAL_PC_SI)       # plot in Mpc, not m.  Note PDF has to change
            pdfvalsPrior = pdfvalsPrior * (1e6*lal.LAL_PC_SI) # rescale units
            pdfvals = pdfvals * (1e6*lal.LAL_PC_SI) # rescale units
        plt.plot(xvals,pdfvalsPrior,label="prior:"+str(param),linestyle='--')
        plt.plot(xvals,pdfvals,label=str(param))
        plt.plot(xvals,cdfvals,label='cdf:'+str(param))
        plt.xlabel(str(param))
        plt.title(titleBase+":"+str(param))
        plt.legend()
    plt.show()

def plotParameterDistributionsFromSamples(titleBase, sampler, ret, keynames):
    nFig = 0
    lnL = np.transpose(ret)[-1] # Assumed.
    for nFig in range(len(keynames)-1):
        param = keynames[nFig]
        plt.figure(nFig)
        plt.clf()
        # Sample distributions
        vals = np.transpose(ret)[nFig]
        if str(param) == "dist":
            fac = (1e6*lal.LAL_PC_SI)
        else:
            fac = 1
        hist, bins = np.histogram(vals/fac, bins=50, density=True, weights=np.exp(lnL))
        center = (bins[:-1]+bins[1:])/2
        plt.plot(center,hist,label="posterior:"+param+":sampled")
        hist, bins  = np.histogram(vals/fac,bins=50,density=True)
        center = (bins[:-1]+bins[1:])/2
        plt.plot(center,hist,label="sampling prior:"+param+":sampled",linestyle='--')
        # Distributions
        xLow = sampler.llim[param]
        xHigh = sampler.rlim[param]
        xvals = np.linspace(xLow,xHigh,500)
        pdfPrior = sampler.prior_pdf[param]
        pdfvalsPrior = pdfPrior(xvals)
        pdf = sampler.pdf[param]
        pdfvals = pdf(xvals)/sampler._pdf_norm[param]
        xvvals = xvals/fac
        pdfvalsPrior = pdfvalsPrior *fac
        pdfvals = pdfvals *fac
        plt.plot(xvals,pdfvalsPrior,label="prior:"+str(param),linestyle='--')
        plt.plot(xvals,pdfvals,label="sampling prior:"+str(param))
        plt.xlabel(str(param))
        plt.title(titleBase+":"+str(param))
        plt.legend()
    plt.show()



#Monica Rizzo, 2018...minor edits by ROS
#
# USAGE
#   python util_ConstructBNSMarginalEOSRank.py --composite G298048/production_C00_cleaned_TaylorT4/all.composite --parameter mc --parameter eta --lnL-cutoff 10 --using-eos ap4

import numpy as np
import matplotlib.pyplot as plt
import argparse

#from lalinference.rapid_pe import RIFT.lalsimutils as lalsimutils
import RIFT.lalsimutils as lalsimutils
import RIFT.physics.EOSManager as EOSManager
#import lalsim_EOS_tools as let
from scipy.integrate import nquad
#import EOS_param as ep
import os

import RIFT.physics.MonotonicSpline as ms 

from sklearn.gaussian_process import GaussianProcessRegressor
from sklearn.gaussian_process.kernels import RBF, WhiteKernel, ConstantKernel as C


###
### Conversion Function
###

#lambda_tilde (this is not implemented how i need it to be implemeted in lalsimutils)
def tidal_lambda_tilde(m1, m2, lam1, lam2):
    
    eta = (m1*m2)/(m1+m2)**2

    lam_til = (8./13.) * ((1 + 7*eta - 31*eta**2) * (lam1 + lam2) \
    + np.sqrt(1 - 4*eta) * (1 + 9*eta - 11*eta**2) * (lam1 - lam2))

    return lam_til

def calc_mc(red_data):
    return lalsimutils.mchirp(red_data[:,1], red_data[:,2])

def calc_eta(red_data):
    return lalsimutils.symRatio(red_data[:,1], red_data[:,2])

def calc_lambda_tilde(red_data):
    return tidal_lambda_tilde(red_data[:,1], red_data[:,2], red_data[:,9], red_data[:,10])

def calc_lambda_from_eos(red_data):

    #calculate lambda values (lambda_of_m defined in )
    lam1 = lambda_of_m(red_data[:,1])
    lam2 = lambda_of_m(red_data[:,2])

    return tidal_lambda_tilde(red_data[:,1], red_data[:,2], lam1, lam2)

#dictionary mapping parameters to functions used to calculate them
param_dict = {'mc': calc_mc, 
              'eta': calc_eta,
              'lambda_tilde': calc_lambda_tilde, 
              'eos_lambda': calc_lambda_from_eos
             }      

###
### Priors
###

#prior functions (most taken from util_ConstructIntrinsincPosterior_GenericCoordinates.py)
def mc_prior(mc):
    return mc/(max_mc - min_mc)

def eta_prior(eta):
    return 1./(eta**(6./5.) * (1.- 4.*eta)**(1/2) * 1.44)

def lambda_tilde_prior(lambda_tilde):
    return 1./5000.

def lambda_from_eos_prior(lam):
    return 1.

def s1z_prior(x):
    return 1./2.

def s2z_prior(x):
    return 1./2.

#dictionary of available priors
prior_dict = {'mc': mc_prior,
              'eta': eta_prior,
              'lambda_tilde': lambda_tilde_prior, 
              'eos_lambda' : lambda_from_eos_prior, 
              's1z': s1z_prior,
              's2z': s2z_prior
             }


###
### Fit Functions
###

#gp fit (basically taken from util_ConstructIntrinsincPosterior_GenericCoordinates.py)
def gp_fit(x, y, mc_index=0):
    
    print "Fitting", len(y), "points: "

    #length scale tuning taken from util_ConstructIntrinsicPosterior_GenericCoordinates.py
    length_scale_est = []
    length_scale_bounds_est = []

    for indx in np.arange(len(x[0])):
    # These length scales have been tuned by experience
        length_scale_est.append(2*np.std(x[:,indx]))  # auto-select range based on sampling retained
        length_scale_min_here= np.max([1e-3,0.2*np.std(x[:,indx]/np.sqrt(len(x)))])
        if indx == mc_index:
            length_scale_min_here= 0.2*np.std(x[:,indx]/np.sqrt(len(x)))
            print " Setting mc range: retained point range is ", np.std(x[:,indx]), " and target min is ", length_scale_min_here
    length_scale_bounds_est.append( (length_scale_min_here, 5*np.std(x[:,indx])) )

    #set up kernel
    kernel = WhiteKernel(noise_level=0.1,noise_level_bounds=(1e-2,1))+C(0.5, (1e-3,1e1))*RBF(length_scale=length_scale_est, length_scale_bounds=length_scale_bounds_est)
    gp = GaussianProcessRegressor(kernel=kernel, n_restarts_optimizer=8)

    #fit and estimate
    gp.fit(x, y)

    def fit_func(coord):
       return gp.predict(coord)[0]
 
    return fit_func


###
### Integral
###

#likelihood integral
def integrate_likelihood(fit_function, bounds, params):

    #inputs: variables corresponding to bounds
    def func(*args):
  
        input_array = []
        for arg in args:
            input_array.append(arg)

        #grab index of mc and eta from params
        if 'eos_lambda' in params:
            mc_indx = params.index('mc')
            eta_indx = params.index('eta')
         
            m1,m2 = lalsimutils.m1m2(args[mc_indx], args[eta_indx])

            lam1 = lambda_of_m(m1)
            lam2 = lambda_of_m(m2)
             
            l = tidal_lambda_tilde(m1, m2, lam1, lam2)

            input_array.append(l)


        #evaluate fit function at input values
        function_value = fit_function([input_array])
     
        #multiply all priors   
        prior_value = 1.
        for p, arg in zip(params, args):
            prior_value *= prior_dict[p](arg)
                                        
        return np.exp(function_value) * prior_value
    
    int_val = nquad(func, bounds)
    
    return int_val

#arguments (obv will need to add more)
parser=argparse.ArgumentParser()
parser.add_argument("--composite-file", type=str, help="file used to fit gp")
parser.add_argument("--parameter", action='append')
parser.add_argument("--using-eos", type=str, default=None, help="Name of EOS if not already determined in lnL")
parser.add_argument("--eos-param", type=str, default=None, help="parameterization of equation of state")
parser.add_argument("--parameter-eos",action='append',default=None,help="Parameters to pass to the EOS model named in 'eos-param'. Specify them as a list, in order: --parameter 1.5 --parameter 0.2 --parameter 3.7 ...")
parser.add_argument("--lnL-cutoff", type=float, default=80.0)
parser.add_argument("--fname-out", type=str, default='integral_result.dat')
opts=parser.parse_args()


#option to be used if gridded values not calculated assuming EOS
if opts.using_eos!=None:
    eos_name=opts.using_eos

    if opts.eos_param == 'spectral':
        # Does not seem to be general -- need to modify 
#        spec_param=ep.make_spec_param_eos(eos_name)
        
#        path=os.getcwd()

        #save for further use        
#        if 'lalsim_eos' not in os.listdir("."):
#            os.mkdir("lalsim_eos")      
        lalsim_spec_param=spec_param/(C_CGS**2)*7.42591549*10**(-25)
        np.savetxt("lalsim_eos/"+eos_name+"_spec_param_geom.dat", np.c_[lalsim_spec_param[:,1], lalsim_spec_param[:,0]])
      
        spec_param_eos=lalsim.SimNeutronStarEOSFromFile(path+"/lalsim_eos/"+eos_name+"_spec_param_geom.dat")
                
        mr_lambda=let.make_mr_lambda(spec_param_eos)

    else:
        my_eos = EOSManager.EOSFromDataFile(name=eos_name,fname =EOSManager.dirEOSTablesBase+"/" + eos_name+".dat")
        mr_lambda=EOSManager.make_mr_lambda(my_eos.eos) 

    lambda_const=ms.interpolate(mr_lambda[:,1], mr_lambda[:,2])    # Precompute constants for interpolation

    #calculate lambda(m)
    def lambda_of_m(mass):  

        if hasattr(mass, '__iter__'):
            lam = np.array([])    
        
            for m in mass:
                # should make ms.interp_func vectorized
                l = ms.interp_func(m, mr_lambda[:,1], mr_lambda[:,2], lambda_const)
                try:
                    if l > 5e3 or  l == None or np.isnan(l):
                        l = 0.
                except:
#                    print " Very unusual lambda(m) situation, probably because of outside the EOS range: ", m, l
                    l=0
                lam = np.append(lam, l)

            return lam
        else:
            lam = ms.interp_func(mass, mr_lambda[:,1], mr_lambda[:,2], lambda_const)
            try:
                if lam > 5e3 or np.isnan(lam) or lam == None:
                    lam = 0.
            except:
 #               print " Very unusual lambda(m) situation, probably because of outside the EOS range: ", mass, lam
                lam=0
            return lam

#    print " Testing lambda(m) function "
#    print lambda_of_m(1.4), lambda_of_m(0.9), lambda_of_m([0.9,1,1.4])

#assume standard composite file (including tides) format
if opts.composite_file:
   
    param_array = []    

    print "Fitting to params:"
    for param in opts.parameter:
        print param
        param_array.append(param)

    #append extra argument to calculate 
    if opts.using_eos:
       param_array.append('eos_lambda')

    #load data 
    comp_data = np.loadtxt(opts.composite_file)

    #determine mc range
    mc_comp = lalsimutils.mchirp(comp_data[:,1], comp_data[:,2]) 
    max_mc = max(mc_comp)
    min_mc = min(mc_comp)

    #set max_lnL
    max_lnL = max(comp_data[:,11])
    lnL_cutoff = opts.lnL_cutoff
 

    #reduce data according to lnL cutoff
    comp_data_reduced = np.array([])

    #apply likelihood cutoff
    for i in range(0, len(comp_data)):
        if comp_data[i,11] >= (max_lnL - lnL_cutoff):
            if len(comp_data_reduced) == 0:
                comp_data_reduced = np.hstack((comp_data_reduced, comp_data[i,:]))
            else:
                comp_data_reduced = np.vstack((comp_data_reduced, comp_data[i,:]))

    #values to fit
    x = np.zeros((len(comp_data_reduced), len(param_array)))
    
    #populate array of fit values
    for i in range(0, len(param_array)):
        x[:,i] = param_dict[param_array[i]](comp_data_reduced)

    lnL = comp_data_reduced[:,11]

    #fit data
    gp_fit_function = gp_fit(x, lnL)

    #print gp_fit_function([[mc[1], eta[1], lambda_tilde[1]]])

    #dictionary of integral bounds corresponding to parameters
    integral_bound_dict = {'mc': [min_mc, max_mc],
                       'eta': [0.14, 0.249999],   # For BNS, we can't go to higher than about 5:1 mass ratio
                       's1z': [-1.0, 1.0],
                       's2z': [-1.0, 1.0]
                      }

    #eta diverges at 0, set a lower bound of 0.01
    #also diverges at 0.25, set to 0.2499999999
 
    #array of integral limits to pass to integral function
    integral_bounds = []
   
    for p in param_array:
        if p != 'eos_lambda':
           integral_bounds.append(integral_bound_dict[p])

    #do the integral
    print "Integrating ..."
    print integral_bounds
    print param_array
    integral_result = integrate_likelihood(gp_fit_function, integral_bounds, param_array)
    print integral_result
    
    np.savetxt(opts.fname_out, integral_result)



#
# spokes.py
# $Id$
#

# GOAL:
#    - library to manage two arrays: *results* and *XML grids*
#    - provides
#         - i/o
#         - consolidating (=joining entries from duplicate masses)
#         - fitting
#         - maximum finding
#         - grid refinement
#
#   - should NOT need NR lookup: works entirely with XML and reported ILE output

import numpy as np
import lal
import RIFT.lalsimutils as lalsimutils
import weight_simulations

rosDebug=False

default_deltaLogL = 10 # 10  # Standard refinement interval in lnL.
n_refine_min =3           # minimum number of points needed for refinement


##
## Spoke fitting
##
def FitSpokeNearPeak(xvals,lnLVals,deltaLogL=default_deltaLogL,refinement_scale_min=0.2,**kwargs):
    # Find the peak
    lnLmax = np.max(lnLVals)
    lnLmin = np.min(lnLVals)
    xmax =np.max(xvals)
    xmin = np.min(xvals)

    # Find x values "sufficiently near" the peak (assume ONE set, for now).  Use top 30%, PLUS anything within 5
    pts_vals = np.array([xvals,np.real(lnLVals)]).T
    pts_sorted = pts_vals[pts_vals[:,1].argsort()] #  np.sort(pts_vals.T,axis=-1)   # usual sort was NOT reliable, oddly!!
#    print " Sorted array ", pts_sorted
    indx_max_base = int(len(pts_sorted)*refinement_scale_min)   # maximum index, based on fraction to keep
    indx_max_delta = np.sum(1 for x in lnLVals if  x > lnLmax-deltaLogL) # len( [x for x,lnL in pts_sorted if lnL> lnLMax-deltaLogL])  # maximum index based on deltaLogL
#    print indx_max_base, indx_max_delta, np.max([indx_max_base,indx_max_delta])
    indx_max = np.max([indx_max_base,indx_max_delta])
    pts_sorted_reduced = pts_sorted[-indx_max:]  # Reduce the number. Minimum length is 3

    xmin_here = np.min(pts_sorted_reduced[:,0])
    xmax_here = np.max(pts_sorted_reduced[:,0])
    z = np.polyfit(pts_sorted_reduced[:,0],pts_sorted_reduced[:,1],2)   # note z[0] is coefficient of x^2, z[2] is coefficient of constant
    if z[0]<0:
        fn = np.poly1d(z)
        def fnReturn(x):
            return np.piecewise(x, [np.logical_and(x>xmin_here, x<xmax_here),np.logical_not(np.logical_and(x>xmin_here , x<xmax_here))] ,[ fn, 0])
        return fnReturn

    if z[0]>0:
        z = np.polyfit(pts_sorted_reduced[:,0],pts_sorted_reduced[:,1],1) 
        fn = np.poly1d(z)
        def fnReturn(x):
            return np.piecewise(x, [np.logical_and(x>xmin_here, x<xmax_here),np.logical_not(np.logical_and(x>xmin_here , x<xmax_here))] ,[ fn, 0])
        return fnReturn
    
    return None

##
## Refinement
##
def Refine(xvals,lnLVals,xmin=None,deltaLogL=default_deltaLogL,npts=10,refinement_scale_min=0.2,**kwargs):
    """
    Refine(xvals,lnLVals) takes a 1d grid of lnL(x) and returns a simple grid refinement guess.
    If the maximum is at or very near an edge (1 or two samples).
    Refinement by default fits to top 30%
    """
    return_code = None
    len_min = 5
    if len(xvals) < len_min:
#        if rosDebug:
        print " FAILED REFINEMENT: Too few points for safety", len(xvals)
        return 'fail', None
    if rosDebug and len(xvals)*refinement_scale_min > npts:
        print " WARNING: Refinement will not increase point density"


    # Find the peak
    # warning: x values are NOT necessarily sorted
    lnLmax = np.max(lnLVals)
    lnLmin = np.min(lnLVals)
    xmax =np.max(xvals)
    xmin = np.min(xvals)
    indx_first = np.argmin(xvals)
    indx_last = np.argmin(xvals)


    # If we are at an edge, extend, with the same resolution as the CLOSEST FEW POINTS
    if lnLVals[indx_first]==lnLmax:
        return_code = 'extended'
        dx = np.std( xvals[-len_min:])/len_min   # reduces chance of numerical catastrophe. Note this is a  large stride
        xvals_new = xmin - np.arange(npts)*dx
        return 'extended',xvals_new
    if lnLVals[indx_last]==lnLmax:
        return_code = 'extended'
        dx = np.std( xvals[-len_min:])/len_min   # reduces chance of numerical catastrophe. Note this is a large stride
        xvals_new = xmax + np.arange(npts)*dx
        return 'extended',xvals_new

    # Find x values "sufficiently near" the peak (assume ONE set, for now).  Use top 30%, PLUS anything within 5
    pts_vals = np.array([xvals,np.real(lnLVals)]).T
    pts_sorted = pts_vals[pts_vals[:,1].argsort()] #  np.sort(pts_vals.T,axis=-1)   # usual sort was NOT reliable, oddly!!
#    print " Sorted array ", pts_sorted
    indx_max_base = int(len(pts_sorted)*refinement_scale_min)   # maximum index, based on fraction to keep
    indx_max_delta = np.sum(1 for x in lnLVals if  x > lnLmax-deltaLogL) # len( [x for x,lnL in pts_sorted if lnL> lnLMax-deltaLogL])  # maximum index based on deltaLogL
    #print indx_max_base, indx_max_delta, np.max([indx_max_base,indx_max_delta])
    indx_max = np.max([n_refine_min,indx_max_base,indx_max_delta])
    pts_sorted_reduced = pts_sorted[-indx_max:]  # Reduce the number. Minimum length is 3

    # Add on elements which are immediately outside the interval, on either side
    #   - important in case the peak is asymmetrically sampled (i.e., only one side of a hill is well-explored)
    xc_min = np.min(pts_sorted_reduced[:,0])
    xc_max = np.min(pts_sorted_reduced[:,0])
    tmp =  pts_sorted[pts_sorted[:,0]<  xc_min ]
    if len(tmp) >0:
        best_before = tmp[-1]
        pts_sorted_reduced = np.append(pts_sorted_reduced,[best_before],axis=0)
    tmp = pts_sorted[pts_sorted[:,0]>  xc_max ]
    if len(tmp)>0:
        best_after = tmp[-1]
        pts_sorted_reduced = np.append(pts_sorted_reduced,[best_after],axis=0)

    # Fail if length too small (should not happen)
    if len(pts_sorted_reduced) < n_refine_min: 
        print "PROBLEM: Reduced length "  # should never happen
        deltaLogLNew = (n_refine_min+1)*1.0/len(xvals) * np.max(lnLVals)  # top few elements are autoselected
        return Refine(xvals,lnLVals, xmin=xmin,deltaLogL=deltaLogLNew,npts=npts,refinement_scale_min=refinement_scale_min)
#        return 'fail', None
    # Return no refinement possible if the array is long (i.e., lnLVals is nearly constant)
#    if len(pts_sorted_reduced) > 0.8*len(xvals):
#        return 'no-refine',None

    # Refine
    # OPTION 1: Quadratic fit refinement
    #    - quadratic fit replacement will not intelligently refine if we are VERY CLOSE to a grid boundary...but it could
    #    - 'refined' grid may be LESS DENSE THAN ORIGINAL GRID
    z = np.polyfit(pts_sorted_reduced[:,0],pts_sorted_reduced[:,1],2)   # note z[0] is coefficient of x^2, z[2] is coefficient of constant
    if z[0]<0:
        fn = np.poly1d(z)
        predicted_xmax = -z[1]/(2*z[0] )
        xmin_here = np.min(pts_sorted_reduced[:,0])
        xmax_here = np.max(pts_sorted_reduced[:,0])
        lnLmin_here =np.min(pts_sorted_reduced[:,1])
        z2 = z - np.array([0,0,np.min([lnLmin_here,lnLmax-deltaLogL])])   # predicted roots, based on 2* delta log L (safety)
        predicted_roots = np.sort(np.roots(z2))
        dx = np.std( xvals[-len_min:])/npts   # reduces chance of numerical catastrophe
        if predicted_roots[0] <xmin_here-len_min*dx:
            predicted_roots[0] = xmin_here
        if predicted_roots[1] > xmax_here+len_min*dx:
            predicted_roots[1] = xmax_here
        if (not (xmin == None)) and predicted_roots[0]<xmin:
            predicted_roots[0] = xmin   # prevent negative total masses
        xvals_new = np.linspace(predicted_roots[0],predicted_roots[1], npts)
        return 'refined', xvals_new
    # OPTION 2: Literal refinement based on surviving points
    xmin_here = np.min(pts_sorted_reduced[:,0])
    xmax_here = np.max(pts_sorted_reduced[:,0])
    return 'refined', np.linspace(xmin_here, xmax_here, npts)


def ChooseWaveformParams_to_spoke_label(P,digits=4):
    m1 = P.m1/lal.MSUN_SI
    m2 = P.m2/lal.MSUN_SI
    s1x = P.s1x
    s1y = P.s1y
    s1z = P.s1z
    s2x = P.s2x
    s2y = P.s2y
    s2z = P.s2z

    line = np.array([m2/m1,s1x,s1y,s1z,s2x,s2y,s2z ])
    return np.around(line, decimals=digits)

def Line_to_spoke_label(line,digits=4):
    m1,m2,s1x,s1y, s1z, s2x,s2y,s2z = line[1:9]
    line = np.array([m2/m1,s1x,s1y,s1z,s2x,s2y,s2z ])
    return np.around(line, decimals=digits)
    
def Line_to_spoke_entry(line,digits=4):
    m1 = line[1]
    m2 = line[2]
    lnL = line[9]
    deltaLogL = line[10]
    return np.around(np.array([m1+m2,lnL,deltaLogL]), decimals=digits)

def ChooseWaveformParams_to_spoke_mass(P,digits=4):
    m1 = P.m1
    m2 = P.m2
    return np.around((m1+m2)/lal.MSUN_SI, decimals=4)


def LoadSpokeDAT(fname):
    # load the *.dat concatenated file 
    dat = np.loadtxt(fname)

    # group by spokes
    sdHere = {}
    for line in dat:
#        spoke_id = str([round(elem, 3) for elem in Line_to_spoke_label(line)])
        spoke_id = str(Line_to_spoke_label(line))
        spoke_contents = Line_to_spoke_entry(line)
        if sdHere.has_key(spoke_id):
            sdHere[spoke_id].append(spoke_contents)
        else:
            sdHere[spoke_id] = [spoke_contents]
    # return 
    return sdHere

def LoadSpokeXML(fname):
    # load the xml file
    P_list = lalsimutils.xml_to_ChooseWaveformParams_array(fname)

    # group by spokes
    sdHere = {}
    for P in P_list:
#        spoke_id = str( [round(elem, 3) for elem in ChooseWaveformParams_to_spoke_label(P)])
        spoke_id = str(ChooseWaveformParams_to_spoke_label(P))
        if sdHere.has_key(spoke_id):
            sdHere[spoke_id].append(P)
        else:
            sdHere[spoke_id] = [P]
    # return 
    return sdHere



##
## Clean spoke entries: remove duplicate masses
##
def CleanSpokeEntries(spoke_entries,digits=4):
    data_at_intrinsic = {}
    # Group entries by their total mass (data in spoke: M, lnL, deltalnL
    for line in spoke_entries:
        mtot, lnL, sigmaOverlnL = line
        if data_at_intrinsic.has_key(mtot):
            data_at_intrinsic[mtot].append( [lnL,sigmaOverlnL])
        else:
            data_at_intrinsic[mtot] = [[lnL,sigmaOverlnL]]

    spoke_entries_out = []
    for key in data_at_intrinsic:
        lnL, sigmaOverL =   np.transpose(data_at_intrinsic[key])
        lnLmax = np.max(lnL)
        sigma = sigmaOverL*np.exp(lnL-lnLmax)  # remove overall Lmax factor, which factors out from the weights constructed from \sigma
        wts = weight_simulations.AverageSimulationWeights(None, None,sigma)   
        lnLmeanMinusLmax = np.log(np.sum(np.exp(lnL - lnLmax)*wts))
        sigmaNetOverL = (np.sqrt(1./np.sum(1./sigma/sigma)))/np.exp(lnLmeanMinusLmax)
        spoke_entries_out.append([key,lnLmeanMinusLmax+lnLmax, sigmaNetOverL])

    return np.around(np.array(spoke_entries_out),decimals=digits)




#! /usr/bin/env python
#
# GOAL
#    - evaluate overlap between two waveforms (two inj xml) in H, L
#    - prints out mismatch in H, L.
#    - no time maximization by default.  (We can do it)
#
# COMPARE TO
#   - util_NRCompareSimulations.py
#   - util_ManualOverlapGrid
#   - util_LALCompareApproximantsOnSamples.py
#
# DEMOS
#     * Compare two approximants with each other, on the same parameters
#     python util_CompareWaveformsInDetectors.py --inj1 inj.xml.gz --inj2 inj.xml.gz --approx SEOBNRv2 --approx2 SEOBNRv4
#     
#    * Compare one single waveform (e.g., NR best fits) with many others (e.g., output of precessing parameters)
#      python util_CompareWaveformsInDetectors.py --inj inj1.xml.gz --inj2 maxpt.xml.gz --approx SEOBNRv2 --group2 Sequence-SXS-All --param2 1 --use-provided-strain2
#
#   * Compare two NR waveforms that have the same parameters

import argparse
import numpy as np

import RIFT.lalsimutils as lalsimutils
import lalsimulation as lalsim
import lal
import sys

import NRWaveformCatalogManager as nrwf
from matplotlib import pyplot as plt


parser = argparse.ArgumentParser()
parser.add_argument("--maximize",action='store_true',help="Perform overlap instead of faithfulness test. Important if inconsistent times")
parser.add_argument("--inj",default=None,type=str,help="Required. Arguments for 1. If used, NR will override intrinsic parameters")
parser.add_argument("--inj2",default=None,type=str,help="Required. Arguments for 2. If used, NR will override intrinsic parameters")
parser.add_argument("--group", default=None,help="inspiral XML file containing injection information.")
parser.add_argument("--param", default="1",help="Parameter value")
parser.add_argument("--group2", default=None,help="inspiral XML file containing injection information.")
parser.add_argument("--param2", default="1",help="Parameter value")
parser.add_argument("--align-22",action='store_true',help='Find the optimal time and phase for alignment for the 22 mode [then halt]')
parser.add_argument("--use-interpolated-overlap",action='store_true', help="The overlap uses a local quadratic approximation near the peak, not just the maximum value")
parser.add_argument("--use-perturbative-extraction", action='store_true')
parser.add_argument("--use-perturbative-extraction-full", action='store_true')
parser.add_argument("--use-provided-strain", action='store_true')
parser.add_argument("--use-provided-strain2", action='store_true')
parser.add_argument("--use-spec-lev",type=int,default=5)
parser.add_argument("--use-hybrid",action='store_true')
parser.add_argument("--use-hybrid-method",default='taper_add')
parser.add_argument("--use-hybrid2",action='store_true')
parser.add_argument("--use-hybrid2-method",default='taper_add')
parser.add_argument("--approx", default="EOBNRv2HM", help="approximant to use for comparison")
parser.add_argument("--approx2", default="EOBNRv2HM", help="approximant to use for comparison")
parser.add_argument("--lmax",default=2,type=int)
parser.add_argument("--srate",type=int,default=16384,help="Sampling rate")
parser.add_argument("--seglen",type=int,default=16,help="Window time")
parser.add_argument("--fref",default=None,type=float,help="Reference frqeuency (assumed equal to fmin)")
parser.add_argument("--fmin",default=20,type=float,help="Minimum frequency for overlap integral -- NOT necessarily tied to tempalte starting frequencies!")
parser.add_argument("--fmin-template1",default=None,type=float,help="Override template frequency 1 (e.g., for NR XML files)")
parser.add_argument("--fmin-template2",default=None,type=float,help="Override template frequency 1 (e.g., for NR XML files)")
parser.add_argument("--fmax",default=2000,type=float,help="Maximum frequency in Hz, used for PSD integral.")
parser.add_argument("--psd-file",default=None,action='append',help="PSD file")
parser.add_argument("--psd",type=str,default="SimNoisePSDaLIGOZeroDetHighPower",help="psd name (attribute in lalsimulation).  SimNoisePSDiLIGOSRD, lalsim.SimNoisePSDaLIGOZeroDetHighPower, lalsimutils.Wrapper_AdvLIGOPsd, .SimNoisePSDiLIGOSRD... ")
parser.add_argument("--fname-output", default="comparison_output.dat",type=str)
parser.add_argument("--verbose", action="store_true",default=False, help="Required to build post-frame-generating sanity-test plots")
parser.add_argument("--save-plots",action="store_true")
parser.add_argument("--tref",default=None,type=float)

opts = parser.parse_args()


###
### Spacing settings, for overlaps
###


lmax = opts.lmax
T_window = opts.seglen
df = 1./T_window
fmin =opts.fmin  # default for now
# if opts.fmin_template1:
#     fmin = np.min([fmin, opts.fmin_template1])
# if opts.fmin_template2:
#     fmin = np.min([fmin, opts.fmin_template2])
fmaxSNR=1700
analyticPSD_Q=True

psd_dict = {}
ifo_list = []

psd=lalsim.SimNoisePSDaLIGOZeroDetHighPower
if opts.psd_file:
    analyticPSD_Q=False

    for inst, psdf in map(lambda c: c.split("="), opts.psd_file):
        if opts.verbose: 
            print "Reading PSD for instrument %s from %s" % (inst, psdf)
        psd_dict[inst] = lalsimutils.load_resample_and_clean_psd(psdf, inst, df)

    ifo_list = psd_dict.keys()

elif opts.psd and hasattr(lalsim, opts.psd):
    ifo_list = ['H1','L1']
    psd = getattr(lalsim, opts.psd)


fNyq = opts.srate/2.
IP=None
IP_list = {}
if opts.maximize:
    for ifo in ifo_list:
        IP_list[ifo]= lalsimutils.ComplexOverlap(fNyq=fNyq,deltaF=df,analyticPSD_Q=analyticPSD_Q,psd=psd_dict[ifo],fMax=opts.fmax,fLow=fmin)
else:
    for ifo in ifo_list:
        IP_list[ifo] = lalsimutils.ComplexIP(fNyq=fNyq,deltaF=df,analyticPSD_Q=analyticPSD_Q,psd=psd_dict[ifo],fMax=opts.fmax,fLow=fmin)

###
### Load injection XML
###

if opts.inj is None:
    print " --inj required"
    sys.exit(0)
print " Reading injection file 1 for comparison ", opts.inj
P1_list = lalsimutils.xml_to_ChooseWaveformParams_array(opts.inj)
nlines1  = len(P1_list)
print " Read  ", nlines1, " injections"

if nlines1 < 1:
    print " No data in ", opts.inj

tref = float( (P1_list[0]).tref ) # default
if not(opts.tref is None):
    tref = opts.tref

if opts.inj2 is None:
    print " --inj2 required"
    sys.exit(0)
print " Reading injection file 1 for comparison ", opts.inj2
P2_list = lalsimutils.xml_to_ChooseWaveformParams_array(opts.inj2)
nlines2  = len(P2_list)

if nlines2 < 1:
    print " No data in ", opts.inj2




###
### Define functions to generate waveforms
###

def get_hF1(indx,ifo):
    global opts
    P = P1_list[indx % nlines1].manual_copy() # looping
    if not (opts.fmin_template1 is None):
        P.fmin = opts.fmin_template1
    if not (opts.fref is None):
        P.fref = opts.fref
    else:
        P.ref = P.fmin
    P.radec =True
    P.tref = P1_list[indx%nlines1].tref # copy, this is an allocated object
    P.deltaF = df
    P.deltaT = 1./opts.srate
    P.detector = ifo
    P.approx = lalsim.GetApproximantFromString(opts.approx)  # override the XML. Can screw you up (ref spins)
    # if P.approx == lalsim.IMRPhenomPv2:
    #     phiJL_now = P.extract_param('phiJL')
    #     P.assign_param('phiJL', phiJL_now-np.pi/2)  # Estimate
#        P.fref = 100
    if opts.verbose:
        P.print_params()
    hF = lalsimutils.non_herm_hoff(P)
    return hF

def get_hF2(indx,ifo):
    global opts
    P = P2_list[indx % nlines2].manual_copy() # looping
    if not (opts.fmin_template2 is None):
        P.fmin = opts.fmin_template2
    if not (opts.fref is None):
        P.fref = opts.fref
    else:
        P.ref = P.fmin
    P.radec =True
    P.tref = P2_list[indx%nlines2].tref # copy, this is an allocated object
    P.deltaF = df
    P.deltaT = 1./opts.srate
    P.detector = ifo
    P.approx = lalsim.GetApproximantFromString(opts.approx2)  # override the XML. Can screw you up
    if P.approx == lalsim.IMRPhenomPv2:
        cosbeta =np.cos(P.extract_param('beta'))
        my_phase = np.pi -np.pi/2
        dt = 20*(P.m1+P.m2)/lal.MSUN_SI * lalsimutils.MsunInSec  # ad hoc factor .. there is apparently a timeshift of Pv2 relative to PD of about 20 M
        P.tref += dt
        phiJL_now = P.extract_param('phiJL')
        psi_now = P.psi
        P.assign_param('phiJL', phiJL_now-my_phase)  # Estimate
        P.psi  = psi_now   # fix L alignment, not J
        P.phiref += my_phase 
        P.phiref += np.pi/2 
    if opts.verbose:
        P.print_params()
    hF = lalsimutils.non_herm_hoff(P)
    return hF


return_hF1 = get_hF1
return_hF2 = get_hF2

###
### Load NR (as needed) and redefine generation functions
###


group = opts.group
param = None
wfP =  None
if group in nrwf.internal_ParametersAreExpressions.keys():
    if not nrwf.internal_ParametersAreExpressions[group]:
        param = opts.param
    else:
        param = eval(str(opts.param))
    if opts.use_spec_lev:
            if group == "Sequence-SXS-All":
                nrwf.internal_FilenamesForParameters[group][param] =nrwf.internal_FilenamesForParameters[group][param].replace("Lev5", "Lev" +str(opts.use_spec_lev))
                print " OVERRIDE OF SXS LEVEL : ", nrwf.internal_FilenamesForParameters[group][param]
    wfP = nrwf.WaveformModeCatalog(opts.group, param, clean_initial_transient=True,clean_final_decay=True, shift_by_extraction_radius=True, lmax=lmax,align_at_peak_l2_m2_emission=True,perturbative_extraction=opts.use_perturbative_extraction,perturbative_extraction_full=opts.use_perturbative_extraction_full,use_provided_strain=opts.use_provided_strain,reference_phase_at_peak=True,quiet=True)
    wfP.P.fmin  = opts.fmin

    print " WARNING: NR orientation parameters differ, so the waveform CANNOT be identical and will generally differ in at least orbital phase "

    def get_hF1_NR(indx,ifo):
        P_here = P1_list[indx % nlines1]

        wfP.P.radec=True
        wfP.P.m1 = P_here.m1
        wfP.P.m2 = P_here.m2

        wfP.P.dist = P_here.dist
        wfP.P.incl = P_here.incl
        wfP.P.theta = P_here.theta
        wfP.P.phi = P_here.phi
        wfP.P.tref = P_here.tref
        wfP.P.psi = P_here.psi
        wfP.P.phiref = P_here.phiref


        wfP.P.fmin = opts.fmin
        wfP.P.deltaF = df
        wfP.P.deltaT = 1./opts.srate
        wfP.P.detector = ifo
        wfP.P.approx = lalsim.GetApproximantFromString(opts.approx)  # override the XML. Can screw you up (ref spins)

        if opts.verbose:
            wfP.P.print_params()

        hF = wfP.non_herm_hoff()
        return hF
    
    return_hF1 = get_hF1_NR


group2 = opts.group2
if group2 in nrwf.internal_ParametersAreExpressions.keys():
    if not nrwf.internal_ParametersAreExpressions[group2]:
        param2 = opts.param2
    else:
        param2 = eval(str(opts.param2))
    # Check for SXS 
    if group2 == "Sequence-SXS-All":
        nrwf.internal_FilenamesForParameters[group2][param2] =nrwf.internal_FilenamesForParameters[group2][param2].replace("Lev5", "Lev" +str(opts.use_spec_lev))
        print " OVERRIDE OF SXS LEVEL : ", nrwf.internal_FilenamesForParameters[group2][param2]
    if opts.verbose:
        print "Importing ", group, param , " and ", group2, param2
    wfP2 = nrwf.WaveformModeCatalog(opts.group2, param2, clean_initial_transient=True,clean_final_decay=True, shift_by_extraction_radius=True, lmax=lmax,align_at_peak_l2_m2_emission=True,perturbative_extraction=opts.use_perturbative_extraction,use_provided_strain=opts.use_provided_strain2,reference_phase_at_peak=True,quiet=True)
    wfP2.P.fmin  = opts.fmin

    print " WARNING: NR orientation parameters differ, so the waveform CANNOT be identical and will generally differ in at least orbital phase "
    def get_hF2_NR(indx,ifo):
        P_here = P2_list[indx % nlines2]

        wfP2.P.radec=True
        wfP2.P.m1 = P_here.m1
        wfP2.P.m2 = P_here.m2

        wfP2.P.dist = P_here.dist
        wfP2.P.incl = P_here.incl
        wfP2.P.theta = P_here.theta
        wfP2.P.phi = P_here.phi
        wfP2.P.tref = P_here.tref
        wfP2.P.psi = P_here.psi
        wfP2.P.phiref = P_here.phiref


        wfP2.P.fmin = opts.fmin
        wfP2.P.deltaF = df
        wfP2.P.deltaT = 1./opts.srate
        wfP2.P.detector = ifo
        wfP2.P.approx = lalsim.GetApproximantFromString(opts.approx2)  # override the XML. Can screw you up (ref spins)
        if opts.verbose:
            wfP2.P.print_params()

        hF = wfP2.non_herm_hoff()

        return hF

    
    return_hF2 = get_hF2_NR








###
### Loop and carry out overlap
###

n_evals = np.max([nlines1,nlines2])

dat_out =[]

for indx in np.arange(n_evals):
  if opts.verbose:
      print indx
  if True:
#  try:
    line = []
    print P1_list[indx].extract_param('thetaJN'), P1_list[indx].phiref, P1_list[indx].extract_param('beta'),
    for ifo in ifo_list: #,'V1']:
        IP = IP_list[ifo]
        hF1 = return_hF1(indx,ifo)
        nm1 = IP.norm(hF1)
        hF2 = return_hF2(indx,ifo)
        nm2 = IP.norm(hF2)
        val = np.abs(IP.ip(hF1,hF2,include_epoch_differences=True)/nm1/nm2)  # correct for timeshift issues, if any
        print " Epoch test ", hF1.epoch, hF2.epoch
        line.append(val)
#        print ifo, IP.ip(hF1,hF2)/nm1/nm2, IP.ip(hF1,hF2),nm1, nm2
        print  val,
        if opts.save_plots:
            if opts.verbose:
                print " --- Saving plot for ", ifo, " ----"
            label1 = opts.approx
            label2 = opts.approx2
            hT1 = lalsimutils.DataInverseFourier(hF1)
            hT2 = lalsimutils.DataInverseFourier(hF2)
            if not (opts.group is None):
                npts = hT1.data.length
                T_wave =npts*hT1.deltaT
                hT1 = lalsimutils.DataRollTime(hT1,-0.5*T_wave/2)
                if opts.verbose:
                    print " Epoch1 ", float(hT1.epoch)
                label1 = group+":"+param
            if not (opts.group2 is None):
                if opts.verbose:
                    print " ---> Rolling to fix FT centering <-- "
                npts = hT2.data.length
                T_wave =npts*hT2.deltaT
                hT2 = lalsimutils.DataRollTime(hT2,-0.5*T_wave/2)
                if opts.verbose:
                    print " Epoch2 ", hT2.epoch
                label2 = group2+":"+param2
            tvals = lalsimutils.evaluate_tvals(hT1) 
            tvals2 = lalsimutils.evaluate_tvals(hT2) 
#            tref = float(hT1.epoch)
            plt.plot(tvals -tref,np.real(hT1.data.data),'r',label=label1)
            plt.plot(tvals2 -tref,np.real(hT2.data.data),'g',label=label2)
            plt.legend()
            plt.savefig(opts.fname_output+"_fig_"+str(indx)+"_"+ifo+".png"); 
            
            # Estimate peak location, so plot can be focused on detail
            tmax_offset =(tvals- tref)[ np.argmax( np.abs(hT1.data.data))]  
                
            plt.xlim(tmax_offset-0.15, tmax_offset+0.05)  # NR scale, focus on merger
            plt.savefig(opts.fname_output+"_fig_"+str(indx)+"_"+ifo+"_detail.png"); 
            plt.clf()
    print
    dat_out.append(line)

#  except:
  else:
      print " Skipping ", indx

np.savetxt(opts.fname_output, np.array(dat_out))



#! /usr/bin/env python
#
# Converts an ILE output (indexed) into STANDARD form, but applying an fmin cut, a sigma cut, and a number of points cut
# Can also use REFERENCE SPIN VALUES
#
# COMPARE TO
#   util_NRBestOfIndex.py
#
# EXAMPLE
#   python ~/research-projects/MonteCarloMarginalizeCode/Code/util_NRIndexedFminCutToILE.py --fname SXS-SingleSim_Ossokine_0234-Lmax2-v2_refine.indexed  --verbose


import argparse
import numpy as np
import RIFT.lalsimutils as lalsimutils
import NRWaveformCatalogManager as nrwf

import lal
MsunInSec = lal.MSUN_SI*lal.G_SI/lal.C_SI**3


###
### Load options
###

parser = argparse.ArgumentParser()
parser.add_argument("--fname", default=None, help="Base output file for ascii text (.dat) and xml (.xml.gz)")
parser.add_argument("--flow",default=30,type=float,help="Use a low frequency cutoff to eliminate points that don't satisfy the necessary condition")
parser.add_argument("--sigma-cut",default=0.95, type=float)
parser.add_argument("--min-npts",default=5e5,type=float)
parser.add_argument("--reference-spins", action='store_true')
parser.add_argument("--verbose", action="store_true",default=False, help="Required to build post-frame-generating sanity-test plots")
opts=  parser.parse_args()



lnLhere  = 0
sigma_here=0
npts_here=0


dat_out = []
with open(opts.fname) as f:
 for line_str in f:
     # string split
     line = line_str.split()
     # Get NR data
     group = line[3]
     if not nrwf.internal_ParametersAreExpressions.has_key(group):
         if opts.verbose:
             print " Cannot parse ",group
         continue
     param = None
     if nrwf.internal_ParametersAreExpressions[group]:
         param = eval(line[4])
     else:
         param = line[4]
     if len(line)<6:
         if opts.verbose:
             print " Line too short"
         continue
     m1 = eval(line[1])
     m2 = eval(line[2])
     try:
         meta_here = nrwf.internal_WaveformMetadata[group][param]
     except:
         print " Skipping ", group, param
         continue
     # in case line abbreviated for some reason
     try:
         lnLhere = float(line[5])  
         sigma_here = float(line[6])
         npts_here = float(line[7])
     except:
         if opts.verbose:
             print " Failed on ", line
         continue
     if sigma_here > opts.sigma_cut:  
         if opts.verbose:
             print " Skipping large error ", line
         continue
     if npts_here < opts.min_npts:
         continue

     try:
         if group == "Sequence-SXS-All":
             Momega0= nrwf.internal_WaveformMetadata[group][param]["Momega0"]
         else:
             Momega0 = nrwf.internal_EstimateStartingMOmega0[group][param]
     except:
         Momega0 = 0
     f0 = Momega0/(m1+m2)/MsunInSec/np.pi  # quadrupole frequency
     
     if True:
     #try:
         wfP = nrwf.WaveformModeCatalog(group,param, metadata_only=True)

         s1x,s1y,s1z = [wfP.P.s1x,wfP.P.s1y,wfP.P.s1z]
         s2x,s2y,s2z = [wfP.P.s2x,wfP.P.s2y,wfP.P.s2z]

         if opts.reference_spins:
             if not 'LAtReference' in nrwf.internal_WaveformMetadata[group][param]:
                 continue
             L = nrwf.internal_WaveformMetadata[group][param]['LAtReference']
             Lhat = L/np.sqrt(np.dot(L,L))
             frmL = lalsimutils.VectorToFrame(Lhat)
             hatX, hatY, hatZ = frmL

             if 'OrbitalPhaseAtReference' in nrwf.internal_WaveformMetadata[group][param]:
                 Phi_ref = nrwf.internal_WaveformMetadata[group][param]['OrbitalPhaseAtReference']
                 hatXp = hatX * np.cos(Phi_ref) + hatY*np.sin(Phi_ref)
                 hatYp = -hatX*np.sin(Phi_ref) + hatY*np.cos(Phi_ref)
                 hatX = hatXp
                 hatY = hatYp
                 # if np.dot(hatX,hatX)>1+1e-5:
                 #     print " Argh, units ", hatX, np.sqrt(np.dot(hatX,hatX)), Phi_ref
                 # if np.dot(hatY,hatY)>1+1e-5:
                 #     print " Argh, units ", hatY, np.sqrt(np.dot(hatY,hatY)), Phi_ref

             if 'Chi1AtReference' in nrwf.internal_WaveformMetadata[group][param]:
                 chi1 = nrwf.internal_WaveformMetadata[group][param]['Chi1AtReference']
                 if np.dot(chi1,chi1) > 1:
                     print " ARRGH ", group,param
                 s1x = np.dot(hatX,chi1)
                 s1y = np.dot(hatY,chi1)
                 s1z = np.dot(hatZ,chi1)
                 if s1x**2 + s1y**2+s1z**2>1:
                     print " ARRGH ", group,param, np.sqrt(s1x**2 + s1y**2+s1z**2), np.sqrt(np.dot(chi1,chi1))
             if 'Chi2AtReference' in nrwf.internal_WaveformMetadata[group][param]:
                 chi2 = nrwf.internal_WaveformMetadata[group][param]['Chi2AtReference']
                 if np.dot(chi2,chi2) > 1:
                     print " ARRGH 2", group,param
                 s2x = np.dot(hatX,chi2)
                 s2y = np.dot(hatY,chi2)
                 s2z = np.dot(hatZ,chi2)
                 if s2x**2 + s2y**2+s2z**2>1:
                     print " ARRGH 2", group,param, np.sqrt(s2x**2 + s2y**2+s2z**2), np.sqrt(np.dot(chi2,chi2))

         if f0 < opts.flow:
             line_out = [ -1, m1, m2, s1x,s1y,s1z, s2x,s2y,s2z, lnLhere, sigma_here,npts_here, float(line[-2])]
             if opts.verbose:
                 print ' '.join(map(str,line_out))
             dat_out.append(line_out)
         else:
             if opts.verbose:
                 print " Skipping line as too high frequency ", f0, line
#     except:
     else:
         print " FAILED TO PARSE ", line
         continue


np.savetxt(opts.fname.replace('indexed', 'composite_cleaned'), np.array(dat_out))




import numpy as np
import lal
import sys

from scipy.optimize import leastsq, brentq
from scipy.linalg import eig, inv
from scipy.interpolate import interp1d

from .. import lalsimutils

rosDebug=True
#from matplotlib import pyplot as plt

def FisherProject(mtx,indx_list_preserve):
    """
    FisherProject(mtx, indx_list_preserve)
    returns mtxAA - mtxAB*(mtxBB^(-1))*mtxBA
    calculated using matrix algebra
    """
    indx_list_drop = np.array(list(set(np.arange(len(mtx)))-set(indx_list_preserve)))
#    print indx_list_preserve, indx_list_drop
    mtxAA = np.matrix(mtx[indx_list_preserve].T[indx_list_preserve])
    mtxBB = np.matrix(mtx[indx_list_drop].T[indx_list_drop])
    mtxAB = np.matrix(mtx[indx_list_preserve].T[indx_list_drop]).T
    # if rosDebug:
    #     print " upper matrix \n", mtxAA, mtxAA.shape
    #     print "lower matrix \n",mtxBB,  mtxBB.shape
    #     print " cross matrix \n", mtxAB,mtxAB.shape
    #     print "returned matrix\n ", mtxOut, mtxOut.shape
    mtxAB*mtxBB
    mtxOut = mtxAA - mtxAB*inv(mtxBB)*(mtxAB.T)
    return mtxOut

# EFFECTIVE FISHER (temporary copy  - should just import from Evan)
#
def effectiveFisher(residual_func, *flat_grids):
    """
    Fit a quadratic to the ambiguity function tabulated on a grid.
    Inputs:
        - a pointer to a function to compute residuals, e.g.
          z(x1, ..., xN) - fit
          for N-dimensions, this is called 'residualsNd'
        - N+1 flat arrays of length K. N arrays for each on N parameters,
          plus 1 array of values of the overlap
    Returns:
        - flat array of upper-triangular elements of the effective Fisher matrix

    Example:
    x1s = [x1_1, ..., x1_K]
    x2s = [x2_1, ..., x2_K]
    ...
    xNs = [xN_1, ..., xN_K]

    gamma = effectiveFisher(residualsNd, x1s, x2s, ..., xNs, rhos)
    gamma
        [g_11, g_12, ..., g_1N, g_22, ..., g_2N, g_33, ..., g_3N, ..., g_NN]
    """
    x0 = np.ones(len(flat_grids))
    fitgamma = leastsq(residual_func, x0=x0, args=tuple(flat_grids))
    return array_to_symmetric_matrix(fitgamma[0])

def residuals2d(gamma, y, x1, x2):
    g11 = gamma[0]
    g12 = gamma[1]
    g22 = gamma[2]
    return y - (1. - g11*x1*x1/2. - g12*x1*x2 - g22*x2*x2/2.)



def array_to_symmetric_matrix(gamma):
    """
    Given a flat array of length N*(N+1)/2 consisting of
    the upper right triangle of a symmetric matrix,
    return an NxN numpy array of the symmetric matrix

    Example:
        gamma = [1, 2, 3, 4, 5, 6]
        array_to_symmetric_matrix(gamma)
            array([[1,2,3],
                   [2,4,5],
                   [3,5,6]])
    """
    length = len(gamma)
    if length==3: # 2x2 matrix
        g11 = gamma[0]
        g12 = gamma[1]
        g22 = gamma[2]
        return np.array([[g11,g12],[g12,g22]])
    if length==6: # 3x3 matrix
        g11 = gamma[0]
        g12 = gamma[1]
        g13 = gamma[2]
        g22 = gamma[3]
        g23 = gamma[4]
        g33 = gamma[5]
        return np.array([[g11,g12,g13],[g12,g22,g23],[g13,g23,g33]])
    if length==10: # 4x4 matrix
        g11 = gamma[0]
        g12 = gamma[1]
        g13 = gamma[2]
        g14 = gamma[3]
        g22 = gamma[4]
        g23 = gamma[5]
        g24 = gamma[6]
        g33 = gamma[7]
        g34 = gamma[8]
        g44 = gamma[9]
        return np.array([[g11,g12,g13,g14],[g12,g22,g23,g24],
            [g13,g23,g33,g34],[g14,g24,g34,g44]])
    if length==15: # 5x5 matrix
        g11 = gamma[0]
        g12 = gamma[1]
        g13 = gamma[2]
        g14 = gamma[3]
        g15 = gamma[4]
        g22 = gamma[5]
        g23 = gamma[6]
        g24 = gamma[7]
        g25 = gamma[8]
        g33 = gamma[9]
        g34 = gamma[10]
        g35 = gamma[11]
        g44 = gamma[12]
        g45 = gamma[13]
        g55 = gamma[14]
        return np.array([[g11,g12,g13,g14,g15],[g12,g22,g23,g24,g25],
            [g13,g23,g33,g34,g35],[g14,g24,g34,g44,g45],[g15,g25,g5,g45,g55]])


# FREQUENCY DOMAIN METHOD
#   - Construct tool to differentiate phase combinations
#     Heavy tool, because it solves ODE for each parameter
#   - Construct tool to weight appropriately (i.e., )


# the tolerances need to scale with the chirp mass, using the correct scaling law
tolerances_relative = { 'mc':1e-3,'mtot': 1e-3}
tolerances_absolute = {'tref':1e-3, 'phiref':0.2, 'chi1': 0.01, 'eta': 0.001, 'theta1': 0.02, 'beta':0.01, 'thetaJN': 0.1 ,'phiJL':0.1}
unit_scale_factors ={'mc': lal.MSUN_SI, 'mtot': lal.MSUN_SI}

def PhaseSequenceLeadingOrder(P):
    """
    Like lalsimutils.PrecessingOrbitModesOfFrequencyApproximate(return_phase=True), but based 
    on leading-order expressions, calculated using V.  
      - *One-sided*.
      - irregular frequency sampling (not identical to what will be generated by that code)
    """
    V, phi, LNvec, E1vec, S1vec, S2vec = PrecessingOrbitOfTime(P)  # Properly handle fref
    freq = np.power(V.data.data,3.)/((P.m1+P.m2)*lsu_G/np.power(lsu_C,3.))/(np.pi)
    tvals = evaluate_tvals(V)  # This seems to be reliably aligned
    PsiF = 3/128./P.extract_parameter('eta') * np.power( freq*np.pi* P.extract_parameter('mc')/lal.MSUN_SI/lalsimutils.MsunInSec, -5./3.)
    alphaF = (2+3.*P.m2/(2*P.m1))*5./32. *(-1./3.)/V.data.data**3   # leading-order power, small spin limit
    betaF = np.ones(len(alphaF))*P.extract_parameter('beta')
    gammaF = alphaF  # small spin limit
    tF = ((P.m1+P.m2)/lal.MSUN_SI* lalsimutils.MsunInSec) * 5./256.* 1./P.extract_parameter('eta')/np.power(V.data.data,8)
    return freq,tF,phi.data.data, alphaF, betaF, gammaF


def PhaseDerivativeSeries(P,p1,skip_fast_derivative=False):
    """
    PhaseDerivativeSeries
    Using the waveform, constructs an array on a *full* range of f values which is nonzero *only for f>0*
    """
    print  " Phase derivatives for ", p1
    Phere = P.manual_copy()
    fvals,fmax_safe, tF,PhiF,alphaF,betaF,gammaF = lalsimutils.PrecessingOrbitModesOfFrequencyApproximate(Phere,2,return_phases=True)
    # zero out all components except that which satisfy conditions. Easiest way to keep on consistent sample
    Psi2F =2*np.pi*fvals*tF - 2*PhiF
    f_pos_bad = np.logical_not(np.logical_and(fvals > 0, fvals<fmax_safe, fvals > P.fmin))
    Psi2F[f_pos_bad] = 0
    alphaF[f_pos_bad]=0
    betaF[f_pos_bad]=0
    gammaF[f_pos_bad]=0

    # For special parameters (time, alpha0, orbital phase), compute the derivatives *manually* via analytic formulae
    if p1 is 'tref' and not skip_fast_derivative:
        # Psi_m  =2 pi f t - m Phi_orb, so derivative is 2 pi f. Others are indepenent of tref
        dPsi2F = 2*np.pi*(fvals)       # from standard reading of tref in form of psi. strictly, dPsi/dtref = omega (dt/dtref) - m dphiorb/dref = omega.  Note that because  of how 
        dPhi2F = np.zeros(len(fvals))   # because of how the calculation is organized
        dalphaF = np.zeros(len(fvals))
        dgammaF = np.zeros(len(fvals))
        return fvals,fmax_safe, dPsi2F, dPsi2F,dalphaF,dgammaF

    if p1 is 'phiJL' and not skip_fast_derivative:
        # dalpha/dp1 = 1, dgamma/dt = cos beta
        dPsi2F = np.zeros(len(fvals))
        dPhi2F = np.zeros(len(fvals))
        dalphaF = np.ones(len(fvals))
        dgammaF = -np.cos(betaF)  # sign
        return fvals,fmax_safe, dPsi2F,dPhi2F,dalphaF,dgammaF

    if p1 is 'phiref' and not skip_fast_derivative:
        dPsi2F = - 2*np.ones(len(fvals))
        dPhi2F = np.ones(len(fvals))
        dalphaF = np.zeros(len(fvals))
        dgammaF = np.zeros(len(fvals))
        return fvals,fmax_safe, dPsi2F,dPhi2F,dalphaF,dgammaF


    # Compute answer via derivative
    p1val = Phere.extract_param(p1)
    if p1 in tolerances_relative.keys():
        dp1 = p1val*tolerances_relative[p1]
    else:
        dp1 = tolerances_absolute[p1]
    if p1 in unit_scale_factors.keys():
        x_scale = unit_scale_factors[p1]
    else:
        x_scale = 1


    # Offset phases
    Phere.assign_param(p1,p1val+dp1)
    fvals_1,fmax_safe_1, tF_1, PhiF_1,alphaF_1,betaF_1,gammaF_1 = lalsimutils.PrecessingOrbitModesOfFrequencyApproximate(Phere,2,return_phases=True)
    Psi2F_1 =2*np.pi*fvals_1*tF_1 - 2*PhiF_1
    f_pos_bad = np.logical_not(np.logical_and(fvals > 0, fvals<fmax_safe, fvals > P.fmin)) # should be unchanged
    Psi2F_1[f_pos_bad] = 0
    alphaF_1[f_pos_bad]=0
    betaF_1[f_pos_bad]=0
    gammaF_1[f_pos_bad]=0


    # Derivative.  Note the series may have different length, so truncate - usually an issue at end, assuming length does not change
    n = len(fvals)
    n1 = len(fvals_1)
    if (1.*n/n1)<0.99 or 1.*n/n1 >1.01:
        print "  Catastrophic length change when computing phase derivative "
        sys.exit(0)
    

    # Derivative of *psi* means we have to form psi first
    # Derivative is *corrected for units* here
    dPsi2F     = (Psi2F_1 -Psi2F)/(dp1/x_scale)
    dPhiF       = (PhiF_1 -PhiF)/(dp1/x_scale)
    dalphaF    = (alphaF_1 - alphaF)/(dp1/x_scale)
    dgammaF  = (gammaF_1 - gammaF)/(dp1/x_scale)

    return fvals,fmax_safe*0.98, dPsi2F,dPhiF, dalphaF,dgammaF  # give a little leeway on frequency




def NumericalFisherMatrixElement(P, p1, p2, psd, max_t=False, **kwargs):
    """
    NumericalFisherMatrixElement
    Full, no-approximations use of <d h | d h>

    NOT RELIABLE -- requires OBSCENELY small step to make deltas of phase resolved.
    """

    Phere = P.manual_copy()
    hf0  = lalsimutils.complex_hoff(Phere)
    if max_t:
        IP = lalsimutils.CreateCompatibleComplexOverlap(hf0, **kwargs)
    else:
        IP = lalsimutils.CreateCompatibleComplexIP(hf0,**kwargs)
    

    # Derivative by p1
    p1val = Phere.extract_param(p1)
    if p1 in tolerances_relative.keys():
        dp1 = p1val*tolerances_relative[p1]
    else:
        dp1 = tolerances_absolute[p1]
    Phere.assign_param(p1,p1val+dp1)
    hf_dp1 = lalsimutils.complex_hoff(Phere)
    Phere.assign_param(p1,p1val)
    hf_dp1.data.data = (hf_dp1.data.data - hf0.data.data)/dp1


    # Derivative by p2
    p2val = Phere.extract_param(p2)
    if p2 in tolerances_relative.keys():
        dp2 = p2val*tolerances_relative[p2]
    else:
        dp2 = tolerances_absolute[p2]
    Phere.assign_param(p2,p2val+dp2)
    hf_dp2 = lalsimutils.complex_hoff(Phere)
    Phere.assign_param(p2,p2val)
    hf_dp2.data.data = (hf_dp2.data.data - hf0.data.data)/dp2


    return IP.ip(hf_dp1,hf_dp2)/IP.norm(hf0)**2


def EffectiveFisherMatrixElement(P,p1,p2,psd,max_t=True,**kwargs):
    if p1 is p2:
        dat =  OverlapVersusChangingParameter(P,p1,psd,max_t=max_t,**kwargs)
        xvals, yvals = dat[:,0], dat[:,1]
        z = np.polyfit(xvals,yvals,2)
        return -0.5*z[0]*np.sign( xvals[-1]-xvals[0])   # coefficient of quadratic
    else:
        print " Off-diagonal elements not yet implemented"
        sys.exit(0)


def ApproximatePrecessingFisherMatrixElement(P, p1,p2,psd,return_intermediate=False,phase_deriv_cache=None,break_out_beta=False,**kwargs):
    rho2Net = 0
    gammaNet= 0
    gammaIntermediate = {}
    rho2Intermediate = {}

    # Precompute all the derivatives I will need - saves some recomputation.  
    # Alternatively, top-level user computes all derivatives once and for all
    if not phase_deriv_cache:
        phase_deriv_cache={}
        phase_deriv_cache[p1] =  PhaseDerivativeSeries(P,p1)
        if not (p1 is p2):
            phase_deriv_cache[p2] =  PhaseDerivativeSeries(P,p2)
    
    for m in [-2,-1,0,1,2]:
        for s in [-1,1]:
            rhoms2, gammaPart = ApproximatePrecessingFisherMatrixElementFactor(P,p1,p2,m,s,psd,phase_deriv_cache=phase_deriv_cache,break_out_beta=break_out_beta,**kwargs)
            if not break_out_beta:
                # adding up these quantities makes NO SENSE if we break up how gamma is computed into parts with different alpha dependence
                rho2Net +=rhoms2
                gammaNet += gammaPart
            gammaIntermediate[(m,s)]=gammaPart
            rho2Intermediate[(m,s)]=rhoms2
#            print " Integral parts ", m,s, rhoms2, gammaPart
#    print " Net amplitude for source ", np.sqrt(rho2Net)
    if return_intermediate:
        return rho2Intermediate, gammaIntermediate  # return the components going into the sum
    else:
        return gammaNet/rho2Net  # return normalized coefficient for this direction, to check overall result


def ApproximatePrecessingFisherMatrixElementFactor(P, p1,p2,m,s,psd,phase_deriv_cache=None,omit_geometric_factor=False,break_out_beta=False,**kwargs):
    """
    ApproximatePrecessingFisherMatrixElementFactor computes the weighted integral appearing in the sum, and the SNR weight.

    Ideally this calculation should CACHE existing phase derivatives (eg. by top-level user)
    Arguments:
       - phase_deriv_cache        : derivative of dphase/d\lambda for different parameters
       - omit_geometric_factor  : 
    """

    # Create basic infrastructure for IP. This requires VERY DENSE sampling...try to avoid
    # hF0 = lalsimutils.complex_hoff(P)
    # IP = lalsimutils.CreateCompatibleComplexIP(hF0,psd=psd,**kwargs)
    # dPsi2F_func =  interp1d(fvals,dPsi2F, fill_value=0, bounds_error=False)
    # dalphaF_func= interp

#    print " called  on ", p1,p2, m, s

    # This geometric stuff is all in common. I should make one call to generate Sh once and for all
    beta_0 = P.extract_param('beta')
    thetaJN_0 = P.extract_param('thetaJN')
    mc_s = P.extract_param('mc')/lal.MSUN_SI *lalsimutils.MsunInSec
    d_s  = P.extract_param('dist')/lal.C_SI  # distance in seconds

    if omit_geometric_factor:
        geometric_factor=1
    else:
        geometric_factor = np.abs(lal.SpinWeightedSphericalHarmonic(thetaJN_0,0,-2,2,int(m)) * lal.WignerDMatrix(2,m,s,0,beta_0,0))**2
    if np.abs(geometric_factor) < 1e-5:
        return 0,0  # saves lots of time

    # Create phase derivatives. Interpolate onto grid?  OR use lower-density sampling
    if phase_deriv_cache:
        if not (p1 in phase_deriv_cache.keys()):
            phase_deriv_cache[p1] =  PhaseDerivativeSeries(P,p1)
        else:
            fvals,fmax_safe, dPsi2F,dPhiF, dalphaF,dgammaF = phase_deriv_cache[p1]            
        if not (p2 in phase_deriv_cache.keys()):
            phase_deriv_cache[p2] =  PhaseDerivativeSeries(P,p2)
        else:
            fvals,fmax_safe, dPsi2F_2,dPhiF_2, dalphaF_2,dgammaF_2 =phase_deriv_cache[p2]            
    else:
        fvals,fmax_safe, dPsi2F,dPhiF, dalphaF,dgammaF = PhaseDerivativeSeries(P,p1)
        fvals,fmax_safe, dPsi2F_2,dPhiF_2, dalphaF_2,dgammaF_2 = PhaseDerivativeSeries(P,p2)

    dropme = np.logical_or(fvals <P.fmin, fvals>0.98*fmax_safe)

    Shvals = np.array(map(psd, np.abs(fvals)))
    Shvals[np.isnan(Shvals)]=float('inf')
    phase_weights =  4* (np.pi*mc_s*mc_s)**2/(3*d_s*d_s) *np.power((np.pi*mc_s*np.maximum(np.abs(fvals),P.fmin/2)),-7./3.)/Shvals  # hopefully one-sided. Note the P.fmin removes nans 
    phase_weights[np.isnan(phase_weights)]=0
    phase_weights[dropme] =0

    dalphaF[dropme] = 0
    dPsi2F[dropme] = 0
    dgammaF[dropme]=0

    dalphaF_2[dropme] = 0
    dPsi2F_2[dropme] = 0
    dgammaF_2[dropme]=0

    rhoms2 = np.sum(phase_weights*P.deltaF)*geometric_factor

    if (beta_0) < 1e-2:  # Pathological, cannot calculate alpha or gamma
        ret_weights = P.deltaF*phase_weights*geometric_factor*( dPsi2F)*(dPsi2F_2)
        ret = np.sum(ret_weights)
    else:
        if not break_out_beta:
            ret_weights = P.deltaF*phase_weights*geometric_factor*( dPsi2F- 2 *dgammaF + m*s*dalphaF)*(dPsi2F_2 - 2*dgammaF_2+m*s*dalphaF_2)
            ret = np.sum(ret_weights)
        else:
            # Warming: this uses the explicit assumption \gamma - -alpha cos beta
            # Warning: you probably never want this unless geometric_factor=1
            if not omit_geometric_factor:
                print " You are extracting a breakdown of the fisher matrix versus beta, but are not fixing  the geometric factor...are you sure?"
            ret_00 = np.sum(P.deltaF*phase_weights*geometric_factor*( dPsi2F)*(dPsi2F_2 ))
            ret_01 = np.sum(P.deltaF*phase_weights*geometric_factor*( dPsi2F)*(dalphaF_2 ))
            ret_10 = np.sum(P.deltaF*phase_weights*geometric_factor*( dalphaF)*(dPsi2F_2 ))
            ret_11 = np.sum(P.deltaF*phase_weights*geometric_factor*( dalphaF)*(dalphaF_2))
            print "  -- submatrix ", p1,p2, ret_00, ret_01, ret_10, ret_11
            return rhoms2,{"00":ret_00, "01":ret_01, "10": ret_10, "11": ret_11}

#    print " Internal fisher element", geometric_factor, rhoms2, ret
    # plt.plot(fvals,ret_weights)
    # plt.ylim(0,np.max(ret_weights))
    # plt.show()
             
    return rhoms2, ret

def OverlapVersusChangingParameter(P,p1,psd,max_t=False,cut_P=0.95,**kwargs):
    """
    Extract <h1|h2>/|h1||h2| as a function of one parameter
    (Intent here is for plotting *or* fitting, to extract diagonal fisher components)
    """
#    if rosDebug:
#        print "Overlap table called with ", kwargs

    Phere = P.manual_copy()
    hf0  = lalsimutils.complex_hoff(Phere)
    if max_t:
        IP = lalsimutils.CreateCompatibleComplexOverlap(hf0, **kwargs)
    else:
        IP = lalsimutils.CreateCompatibleComplexIP(hf0,**kwargs)
    

    
    p1val = Phere.extract_param(p1)
    if p1 in tolerances_relative.keys():
        dp1 = p1val*tolerances_relative[p1]
    else:
        dp1 = tolerances_absolute[p1]
    if p1 in unit_scale_factors.keys():
        x_scale = unit_scale_factors[p1]
    else:
        x_scale = 1

    xvals  = np.linspace(p1val-dp1,p1val+dp1,20)
    dat = []
    for x in xvals:
        Phere.assign_param(p1,x)
#        if not Phere.SoftAlignedQ():
#            Phere.print_params(show_system_frame=True)
        hF= lalsimutils.complex_hoff(Phere)
        z = IP.ip(hf0,hF)/IP.norm(hf0)/IP.norm(hF)
        if z > cut_P:
            dat.append( [x/x_scale,z])  # NORMALIZED

    return np.array(dat)

def OverlapGridVersusChangingParameter(P,p1,p2,psd,max_t=False,cut_P=0.95,**kwargs):
    """
    Extract <h1|h2>/|h1||h2| as a function of one parameter
    (Intent here is for plotting *or* fitting, to extract diagonal fisher components)
    """
#    if rosDebug:
#        print "Overlap table called with ", kwargs

    Phere = P.manual_copy()
    hf0  = lalsimutils.complex_hoff(Phere)
    if max_t:
        IP = lalsimutils.CreateCompatibleComplexOverlap(hf0, **kwargs)
    else:
        IP = lalsimutils.CreateCompatibleComplexIP(hf0,**kwargs)
    
    p1val = Phere.extract_param(p1)
    p2val = Phere.extract_param(p2)
    if p1 in tolerances_relative.keys():
        dp1 = p1val*tolerances_relative[p1]
    else:
        dp1 = tolerances_absolute[p1]
    if p2 in tolerances_relative.keys():
        dp2 = p2val*tolerances_relative[p2]
    else:
        dp2 = tolerances_absolute[p2]

    xvals  = np.linspace(p1val-dp1,p1val+dp1,9)
    yvals  = np.linspace(p2val-dp2,p2val+dp2,9)
    dat =[]
    if p1 in unit_scale_factors.keys():
        x_scale = unit_scale_factors[p1]
    else:
        x_scale = 1
    if p2 in unit_scale_factors.keys():
        y_scale = unit_scale_factors[p2]
    else:
        y_scale = 1
    for x in xvals:
     for y in yvals:
        Phere.assign_param(p1,x)
        Phere.assign_param(p2,y)
#        print (x/x_scale,y/y_scale), (Phere.extract_param(p1)/x_scale, Phere.extract_param(p2)/y_scale)
        hF= lalsimutils.complex_hoff(Phere)
        z = IP.ip(hf0,hF)/IP.norm(hf0)/IP.norm(hF)  # NORMALIZED
        if z> cut_P:
            dat.append([x/x_scale,y/y_scale,z])

    return np.array(dat)
